/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */



/*------------------------------------------------------------------------*/
/*--									--*/
/*--	ADM_ParseCmdLine.c						--*/
/*--									--*/
/*--	Parse command line arguments					--*/
/*------------------------------------------------------------------------*/

#include "ITTC.h"

int ADM_ParseCmdLine(ADM_CTX *ctx, int argc, char *argv[])
{
  int i;
  char *c,*d;
  int helpflag=0;

  /*--  Initialize command line arguments structure  --*/

  ctx->opcode = -1;
  ctx->keysize = NULL;
  ctx->ConfigFile = CTL_CONFIG_FILE;
  ctx->keyname = NULL;
  ctx->password = NULL;
  ctx->threshold = 0;
  ctx->numServers = 0;
  ctx->server = -1;
  ctx->me = 0;

  /*--  Parse command line arguments  --*/

  for (i=1; i<argc; ++i) {

    c = argv[i];

    d = argv[i+1];

    if (strcmp(c, "-h") == 0)
      helpflag = 1;

    if (strcmp(c, "-c") == 0) {
      ctx->ConfigFile = d;
      i++;
    }

    if (strcmp(c, "-gen") == 0) {
      ctx->opcode = ADM_op_genkey;
      ctx->keysize = d;
      ctx->threshold = argv[i+2][0] - '0';
      ctx->keyname = argv[i+3];
      i+=3;
    }

    if (strcmp(c, "-n") == 0) {
      ctx->numServers = d[0] - '0';
      i++;
    }

    if ( (strcmp(c, "-ID") == 0) || (strcmp(c, "-id") == 0) ) {
      ctx->me = d[0] - '0';
      i++;
    }

    if (strcmp(c, "-shutdown") == 0) {
      ctx->opcode = ADM_op_shutdown;
      if ( (i+1 < argc) && (d[0] != '-') ) {
	ctx->server = d[0] - '0';
	i++;
      }
    }

    if (strcmp(c, "-refresh") == 0) {
      ctx->opcode = ADM_op_refresh;
      ctx->keyname = d;
      i++;
    }

    if (strcmp(c, "-password") == 0) {
      ctx->password = d;
      i++;
    }

    if (strcmp(c, "-start") == 0) {
      ctx->opcode = ADM_op_start;
      if ( (i+1 < argc) && (d[0] != '-') ) {
	ctx->server = d[0] - '0';
	i++;
      }
    }

    if (strcmp(c, "-suspend") == 0) {
      ctx->opcode = ADM_op_suspend;
      if ( (i+1 < argc) && (d[0] != '-') ) {
	ctx->server = d[0] - '0';
	i++;
      }
    }

    if (strcmp(c, "-corrupt") == 0) {
      ctx->opcode = ADM_op_corrupt;
      if ( (i+1 < argc) && (d[0] != '-') ) {
	ctx->server = d[0] - '0';
	i++;
      }
    }

  }

  if (ctx->opcode < 0 || helpflag) {
    printf("\n");
    printf("======================================================================\n");
    printf("  admittc: administer ITTC daemons. Usage:\n");
    printf("----------------------------------------------------------------------\n");
    printf("Usage:  admittc args\n");
    printf("  -start [server]       Start specified server.  Default: all servers.\n");
    printf("  -shutdown [server]    Shutdown specified server.\n");
    printf("  -suspend [server]     Suspend specified server.\n");
    printf("                         Use -start to un-suspend server\n");
    printf("  -corrupt [server]     Make server behave as corrupt server.\n");
    printf("                        Use -start to uncorrupt server\n");
    printf("......................................................................\n");
    printf("  -gen key-length threshold key-name     \n");
    printf("               Generate a new key of specified length \n");
    printf("               and threshold.\n");
    printf("               key-length: Strong = 2048-bits\n");
    printf("                           Normal = 1024 bits\n");
    printf("                           Weak =  512 bits\n");
    printf("                           Test =  128 bits\n");
    printf("  -refresh key-name     Refresh sharing of specified key.\n");
    printf("  -------------------\n");
    printf("  -c file-name          Config file name.\n");
    printf("  -password pwd         Password used to decrypt private shares.\n");
    printf("  -n num                Total number of share servers.\n");
    printf("  -id admin-id          (default = 0)\n");
    printf("  -h                    Prints usage instructions.\n");
    printf("----------------------------------------------------------------------\n");
    printf("Examples:\n");
    printf("  To issue a start command to all server:\n");
    printf("     admittc -c Config -password hhhhhh -n 3 -start\n");
    printf("\n");
    printf("  To suspend server number 1:\n");
    printf("     admittc -c Config -n 3 -suspend 1\n");
    printf("\n");
    printf("  To generate a new 2-out-of-3 shared key called key1:\n");
    printf("     admittc -c Config -n 3 -gen Normal 2 key1\n");
    printf("\n");
    printf("  To refresh the key key2:\n");
    printf("        admittc -c Config -n 3 -refresh key2\n");
    printf("----------------------------------------------------------------------\n");
    printf("Web Interface:\n");
    printf("  To use the web interface, first run the adm web server using:\n");
    printf("     perl adm.pl\n");
    printf("  Then load adm.html in a web browser.\n");
    printf("======================================================================\n\n");
    return RC_ERROR;
  }

  if ( (ctx->opcode == ADM_op_start) && (ctx->password == NULL) ) {
    printf("admittc: start command requires password.\n\n");
    return RC_ERROR;
  }

  if (ctx->numServers <= 0){
    printf("admittc: must set number of servers to be greater than 0.\n\n");
    return RC_ERROR;
  }

  if ( (ctx->opcode == ADM_op_genkey) && 
       ((ctx->keysize == NULL) || (ctx->keyname == NULL))) {
    printf("admittc: -gen opcode requires key size and key name.\n\n");
    return RC_ERROR;
  }

  if ( (ctx->opcode == ADM_op_refresh) && 
       (ctx->keyname == NULL)) {
    printf("admittc: -refresh opcode requires key name.\n\n");
    return RC_ERROR;
  }

  return RC_OK;
}
