/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

/* Clear transport layer - COM package */

#ifndef WIN32
#include <unistd.h>
#include <sys/socket.h>
#endif
#include <stdlib.h>
#include <stdio.h>

#include "COM.h"

extern PACKAGE FILE * errfp;

static TRANS_CTX *
COM_clear_new_clisrv_ctx(magic, num)
     TRANS_MAGIC * magic;
     int num;
{
  return (TRANS_CTX *) 1;
}

static void
COM_clear_close_ctx(ctx)
     TRANS_CTX * ctx;
{
}

static TRANS_STREAM *
COM_clear_new_clisrv_stream(ctx, fd)
     TRANS_CTX * ctx;
{
  int * x;

  if((x = malloc(sizeof(int))) == NULL)
    return NULL;
  *x = fd;
  return (TRANS_STREAM *) x;
}

static int
COM_clear_handshake_status(stream)
     TRANS_STREAM * stream;
{
  return 0;
}

static int
COM_clear_do_handshake(stream)
     TRANS_STREAM * stream;
{
  return 1;
}

static char *
COM_clear_get_id(stream)
     TRANS_STREAM * stream;
{
  return NULL;
}

static int
COM_clear_read(stream, buf, len)
     TRANS_STREAM * stream;
     unsigned char * buf;
     int len;
{
  int ret = recv(*(int *) stream, buf, len, 0);

#ifdef WIN32
  if(ret < 0 && WSAGetLastError() == WSAECONNRESET)
    return 0;
#endif
  return ret;
}

static int
COM_clear_write(stream, buf, len)
     TRANS_STREAM * stream;
     unsigned char * buf;
     int len;
{
  return send(*(int *) stream, buf, len, 0);
}

static void
COM_clear_close_stream(stream, shutdown)
     TRANS_STREAM * stream;
     int shutdown;
{
#ifdef WIN32
  closesocket(*(int *) stream);
#else
  close(*(int *) stream);
#endif
  free(stream);
}

static void
COM_clear_error(where)
     char * where;
{
  perror(where);
}

static COM_TRANSPORT clear_trans = {
  COM_clear_new_clisrv_ctx,
  COM_clear_new_clisrv_ctx,
  COM_clear_close_ctx,
  COM_clear_new_clisrv_stream,
  COM_clear_new_clisrv_stream,
  COM_clear_handshake_status,
  COM_clear_do_handshake,
  COM_clear_get_id,
  COM_clear_read,
  COM_clear_write,
  COM_clear_close_stream,
  COM_clear_error,
  NULL
};

COM_TRANSPORT *
COM_clear_transport()
{
  return &clear_trans;
}
