/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

/* Communications transport glue - COM package */

#include <stdio.h>

#include "COM.h"

static struct {
  char * t_name;
  COM_TRANSPORT * (* t_transfunc)();
} trans_table[] = {
  { "SSL", COM_SSL_transport },
  { "sslv3", COM_SSL_transport },
  { "clear", COM_clear_transport },
  { NULL, NULL }
};

COM_TRANSPORT *
COM_transport_lookup(name)
     char * name;
{
  int i = 0;

  if(name == NULL)
    return NULL;
  while(trans_table[i].t_name)
    if(!strcmp(trans_table[i].t_name, name))
      return (*trans_table[i].t_transfunc)();
    else
      ++i;
  return NULL;
}

TRANS_CTX *
COM_new_client_transport_ctx(trans, magic, num)
     COM_TRANSPORT * trans;
     TRANS_MAGIC * magic;
     int num;
{
  return (* trans->new_client_ctx_method)(magic, num);
}

TRANS_CTX *
COM_new_server_transport_ctx(trans, magic, num)
     COM_TRANSPORT * trans;
     TRANS_MAGIC * magic;
     int num;
{
  return (* trans->new_server_ctx_method)(magic, num);
}

void
COM_close_transport_ctx(trans, ctx)
     COM_TRANSPORT * trans;
     TRANS_CTX * ctx;
{
  (* trans->close_ctx_method)(ctx);
}

TRANS_STREAM *
COM_new_transport_client_stream(trans, ctx, fd)
     COM_TRANSPORT * trans;
     TRANS_CTX * ctx;
     int fd;
{
  return (* trans->new_client_stream_method)(ctx, fd);
}

TRANS_STREAM *
COM_new_transport_server_stream(trans, ctx, fd)
     COM_TRANSPORT * trans;
     TRANS_CTX * ctx;
     int fd;
{
  return (* trans->new_server_stream_method)(ctx, fd);
}

int
COM_transport_handshake_status(trans, stream)
     COM_TRANSPORT * trans;
     TRANS_STREAM * stream;
{
  if(stream == NULL)
    return -1;
  return (* trans->handshake_stat_method)(stream);
}

int
COM_transport_do_handshake(trans, stream)
     COM_TRANSPORT * trans;
     TRANS_STREAM * stream;
{
  if(stream == NULL)
    return -1;
  return (* trans->do_handshake_method)(stream);
}

char *
COM_transport_get_id(trans, stream)
     COM_TRANSPORT * trans;
     TRANS_STREAM * stream;
{
  if(stream == NULL)
    return NULL;
  return (* trans->get_id_method)(stream);
}

int
COM_transport_read(trans, stream, buf, len)
     COM_TRANSPORT * trans;
     TRANS_STREAM * stream;
     unsigned char * buf;
     int len;
{
  if(stream == NULL)
    return 0;
  return (* trans->read_method)(stream, buf, len);
}

int
COM_transport_write(trans, stream, buf, len)
     COM_TRANSPORT * trans;
     TRANS_STREAM * stream;
     unsigned char * buf;
     int len;
{
  if(stream == NULL)
    return -1;
  return (* trans->write_method)(stream, buf, len);
}

void
COM_transport_close_stream(trans, stream, shutdown)
     COM_TRANSPORT * trans;
     TRANS_STREAM * stream;
     int shutdown;
{
  if(stream)
    (* trans->close_stream_method)(stream, shutdown);
}

void
COM_transport_handle_error(trans, where)
     COM_TRANSPORT * trans;
     char * where;
{
  (* trans->error_method)(where);
}
