/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

/***************************************************************/
/*                                                             */
/* GEN_TrialDivision.c                                         */
/*                                                             */
/***************************************************************/

#include "ITTC.h"

/*------------------------------------------------------------------------*/
/*--									--*/
/*--	GEN_RelPrime.c							--*/
/*--									--*/
/*--	Takes a BIGNUM and an int.  Returns 1 if they're relatively	--*/
/*--	prime, a 0 if they're not, and a -1 on an error.		--*/
/*--									--*/
/*--	int gcd(x, y) { return y==0 ? x : gcd(y, x%y); }		--*/
/*--									--*/
/*------------------------------------------------------------------------*/

int GEN_RelPrime(BIGNUM *bn, int n) {

  unsigned long int x, y, z;


  /*------------------------------------------------------------------*/
  /* Start GCD - The first step is to find the bignum mod the integer */
  /*------------------------------------------------------------------*/

  if ( (n == 0) || (BN_is_zero(bn)) ) {
      UTL_Log("GEN_RelPrime", LOG_INFO, "Can't find GCD of a number and zero.");
      return RC_ERROR;
  }

  x = n;
  y = BN_mod_word(bn, n);


  /*----------------------------------------*/
  /* Now do the rest of the GCD on integers */
  /*----------------------------------------*/

  while (y != 0) {

    z = x % y;
    x = y;
    y = z;

  }

  return (x == 1);
}



/* Trial Division */
int GEN_TrialDivision(int *failed, BIGNUM *N, GEN_CTX *genCtx) {

  int mme;
  int numServers;
  int i;
  int numFactors;
  unsigned long int *factors = NULL;


  /*------------*/
  /* Initialize */
  /*------------*/

  *failed = 0;
  mme = GEN_MyID(genCtx);
  numServers = GEN_NumServers(genCtx);
  factors = GEN_TrialDivFactors(genCtx);
  numFactors = GEN_NumTrialDivFactors(genCtx);
  

  /*--------------------------*/
  /* Start the trial division */
  /*--------------------------*/

  for (i=mme; i<numFactors; i+=numServers) {
    
    if (GEN_RelPrime(N, factors[i])) {
      *failed = 0;
    } else {
      *failed = 1;
      return RC_OK;
    }
  }
       
  /* Return the error code */
  return RC_OK;
}

