/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

#include <stdlib.h>

#include <openssl/crypto.h>
#include <openssl/err.h>
#include <openssl/asn1_mac.h>
#include <openssl/pem.h>

#include "KEY.h"

static RSA_SHARE *
d2i_RSAShare(a,pp,length)
RSA_SHARE **a;
unsigned char **pp;
long length;
	{
	int i=ASN1_R_PARSING, j;
	ASN1_INTEGER *bs=NULL;
	RSA_SHARE_APP_DATA * data;
	M_ASN1_D2I_vars(a,RSA_SHARE *,KEY_RSA_share_new);

	M_ASN1_D2I_Init();
	M_ASN1_D2I_start_sequence();
	M_ASN1_D2I_get(bs,d2i_ASN1_INTEGER);
	if (bs->length == 0)
		ret->version=0;
	else	ret->version=bs->data[0];
	M_ASN1_D2I_get(bs,d2i_ASN1_INTEGER);
	if ((ret->n=BN_bin2bn(bs->data,bs->length,ret->n)) == NULL) goto err_bn;
	M_ASN1_D2I_get(bs,d2i_ASN1_INTEGER);
	if ((ret->e=BN_bin2bn(bs->data,bs->length,ret->e)) == NULL) goto err_bn;

	ret->d = NULL;
	ret->p = NULL;
	ret->q = NULL;
	ret->dmp1 = NULL;
	ret->dmq1 = NULL;
	ret->iqmp = NULL;

	data = KEY_RSA_share_get_data(ret);
	M_ASN1_D2I_get(bs,d2i_ASN1_INTEGER);
	if ((data->nparties = ASN1_INTEGER_get(bs)) < 0)
	  goto err_bn;
	M_ASN1_D2I_get(bs,d2i_ASN1_INTEGER);
	if ((data->threshold = ASN1_INTEGER_get(bs)) < 0)
	  goto err_bn;
	M_ASN1_D2I_get(bs,d2i_ASN1_INTEGER);
	if ((data->ver = ASN1_INTEGER_get(bs)) < 0)
	  goto err_bn;
	M_ASN1_D2I_get(bs,d2i_ASN1_INTEGER);
	if ((data->k = ASN1_INTEGER_get(bs)) < 0 || data->k > MAX_RSA_SHARING)
	  goto err_bn;

	for(j = 0; j < data->k; ++j) {
	  M_ASN1_D2I_get(bs,d2i_ASN1_INTEGER);
	  if ((data->d[j]=BN_bin2bn(bs->data,bs->length,data->d[j])) == NULL)
	    goto err_bn;
	  if(bs->type == V_ASN1_NEG_INTEGER)
	    data->d[j]->neg = 1;
	}

	ASN1_INTEGER_free(bs);
	bs = NULL;

	M_ASN1_D2I_Finish_2(a);
err_bn:
	i=ERR_R_BN_LIB;
err:
	ASN1err(ASN1_F_D2I_RSAPRIVATEKEY,i);
	if ((ret != NULL) && ((a == NULL) || (*a != ret))) RSA_free(ret);
	if (bs != NULL) ASN1_INTEGER_free(bs);
	return(NULL);
	}

static RSA_SHARED_KEY *
d2i_RSASharedKey(a,pp,length)
RSA_SHARED_KEY **a;
unsigned char **pp;
long length;
	{
	int i=ASN1_R_PARSING, j;
	ASN1_INTEGER *bs=NULL;
	RSA_SHARED_KEY_APP_DATA * data;
	M_ASN1_D2I_vars(a,RSA_SHARED_KEY *,KEY_RSA_shared_key_new);

	M_ASN1_D2I_Init();
	M_ASN1_D2I_start_sequence();
	M_ASN1_D2I_get(bs,d2i_ASN1_INTEGER);
	if (bs->length == 0)
		ret->version=0;
	else	ret->version=bs->data[0];
	M_ASN1_D2I_get(bs,d2i_ASN1_INTEGER);
	if ((ret->n=BN_bin2bn(bs->data,bs->length,ret->n)) == NULL) goto err_bn;
	M_ASN1_D2I_get(bs,d2i_ASN1_INTEGER);
	if ((ret->e=BN_bin2bn(bs->data,bs->length,ret->e)) == NULL) goto err_bn;

	ret->d = NULL;
	ret->p = NULL;
	ret->q = NULL;
	ret->dmp1 = NULL;
	ret->dmq1 = NULL;
	ret->iqmp = NULL;

	data = KEY_RSA_shared_key_get_data(ret);

	M_ASN1_D2I_get(bs,d2i_ASN1_INTEGER);
	if ((data->nparties = ASN1_INTEGER_get(bs)) < 0)
	  goto err_bn;
	M_ASN1_D2I_get(bs,d2i_ASN1_INTEGER);
	if ((data->threshold = ASN1_INTEGER_get(bs)) < 0)
	  goto err_bn;

	M_ASN1_D2I_get(bs,d2i_ASN1_INTEGER);
	if ((data->g=BN_bin2bn(bs->data,bs->length,data->g)) == NULL) goto err_bn;

	M_ASN1_D2I_get(bs,d2i_ASN1_INTEGER);
	if ((data->k = ASN1_INTEGER_get(bs)) < 0 || data->k > MAX_RSA_SHARING)
	  goto err_bn;

	for(j = 0; j < data->k; ++j) {
	  M_ASN1_D2I_get(bs,d2i_ASN1_INTEGER);
	  if ((data->gd[j]=BN_bin2bn(bs->data,bs->length,data->gd[j])) == NULL)
	    goto err_bn;
	  if(bs->type == V_ASN1_NEG_INTEGER)
	    data->gd[j]->neg = 1;
	}

	ASN1_INTEGER_free(bs);
	bs = NULL;

	M_ASN1_D2I_Finish_2(a);
err_bn:
	i=ERR_R_BN_LIB;
err:
	ASN1err(ASN1_F_D2I_RSAPRIVATEKEY,i);
	if ((ret != NULL) && ((a == NULL) || (*a != ret))) RSA_free(ret);
	if (bs != NULL) ASN1_INTEGER_free(bs);
	return(NULL);
	}

int
i2d_RSAShare(a,pp)
RSA_SHARE *a;
unsigned char **pp;
	{
	BIGNUM *num[MAX_RSA_SHARING+7];
	BIGNUM *plus1;
	ASN1_INTEGER bs;
	unsigned int k,i,tot,t,len,max=0;
	unsigned char *p;
	RSA_SHARE_APP_DATA * d;

	if (a == NULL) return(0);

	if ((d = KEY_RSA_share_get_data(a)) == NULL) return(0);

	num[1]=a->n;
	num[2]=a->e;
	num[3]=BN_new();
	BN_set_word(num[3], d->nparties);
	num[4]=BN_new();
	BN_set_word(num[4], d->threshold);
	num[5]=BN_new();
	BN_set_word(num[5], d->ver);
	num[6]=BN_new();
	BN_set_word(num[6], d->k);

	for(k = 0; k < d->k; ++k)
	  num[k+7]=d->d[k];

	for (i=1; i<d->k+7; i++) {
	  len = BN_num_bits(num[i]) / 8;
	  if(len > max)
	    max = len;
	}

	bs.data=(unsigned char *)Malloc(max+4);
	if (bs.data == NULL)
		{
		ASN1err(ASN1_F_I2D_RSAPRIVATEKEY,ERR_R_MALLOC_FAILURE);
		return(-1);
		}

	bs.length=1;
	bs.type=V_ASN1_INTEGER;
	bs.data[0]=a->version&0x7f;

	tot=i2d_ASN1_INTEGER(&(bs),NULL);
	plus1 = BN_new();
	for (i=1; i<d->k+7; i++)
		{
		if(num[i]->neg) {
		bs.length=BN_bn2bin(num[i],bs.data);
		bs.type=V_ASN1_NEG_INTEGER;
		}
		else {
		bs.length=BN_bn2bin(num[i],bs.data);
		bs.type=V_ASN1_INTEGER;
		}
		len = i2d_ASN1_INTEGER(&bs,NULL);
		if (len > max) max=len;
		tot+=len;
		}

	BN_clear_free(plus1);

	t=ASN1_object_size(1,tot,V_ASN1_SEQUENCE);
	if (pp == NULL) return(t);

	p= *pp;
	ASN1_put_object(&p,1,tot,V_ASN1_SEQUENCE,V_ASN1_UNIVERSAL);

	bs.length=1;
	bs.type=V_ASN1_INTEGER;
	bs.data[0]=a->version&0x7f;

	i2d_ASN1_INTEGER(&bs,&p);

	for (i=1; i<d->k+7; i++)
		{
		if(num[i]->neg) {
		bs.length=BN_bn2bin(num[i],bs.data);
		bs.type=V_ASN1_NEG_INTEGER;
		}
		else {
		bs.length=BN_bn2bin(num[i],bs.data);
		bs.type=V_ASN1_INTEGER;
		}
		i2d_ASN1_INTEGER(&bs,&p);
		}
	/* TODO: do cleanup on error properly */
	BN_clear_free(num[3]);
	BN_clear_free(num[4]);
	BN_clear_free(num[5]);
	BN_clear_free(num[6]);
	Free((char *)bs.data);
	*pp=p;
	return(t);
	}

int
i2d_RSASharedKey(a,pp)
RSA_SHARED_KEY *a;
unsigned char **pp;
	{
	BIGNUM *num[MAX_RSA_SHARING+7];  /* TJW:May need to increase size */
	BIGNUM *plus1;
	unsigned char data[1];
	ASN1_INTEGER bs;
	unsigned int k,j,i,tot,t,len,max=0;
	unsigned char *p;
	RSA_SHARED_KEY_APP_DATA * d;

	if (a == NULL) return(0);

	if ((d = KEY_RSA_shared_key_get_data(a)) == NULL) return(0);

	num[1]=a->n;
	num[2]=a->e;
	num[3]=BN_new();
	BN_set_word(num[3], d->nparties);
	num[4]=BN_new();
	BN_set_word(num[4], d->threshold);
	num[5]=d->g;
	num[6]=BN_new();
	BN_set_word(num[6], d->k);

	for(k = 0; k < d->k; ++k)
	  num[k+7]=d->gd[k];

	bs.length=1;
	bs.data=data;
	bs.type=V_ASN1_INTEGER;
	data[0]=a->version&0x7f;

	tot=i2d_ASN1_INTEGER(&(bs),NULL);
	plus1 = BN_new();
	for (i=1; i<d->k+7; i++)
		{
		if(num[i]->neg) {
		  BN_add(plus1, num[i], BN_value_one());
		  j=BN_num_bits(plus1);
		  len=((j == 0)?1:((j/8)+1));
		  /* This requires the bignum package to be able to
		   * calculate number of bits EXACTLY.  We want to
		   * determine if the MSB is 1, which happens when
		   * the bit length (in BN) is 1 (mod 8). */
		  if(BN_num_bits(num[i])%8 == 1)
		    ++len;
		  if (len > max) max=len;
		  len=ASN1_object_size(0,len,V_ASN1_INTEGER);
		}
		else {
		  j=BN_num_bits(num[i]);
		  len=((j == 0)?1:((j/8)+1));
		  if (len > max) max=len;
		  len=ASN1_object_size(0,len,V_ASN1_INTEGER);
		}

		tot+=len;
		}

	BN_clear_free(plus1);

	t=ASN1_object_size(1,tot,V_ASN1_SEQUENCE);
	if (pp == NULL) return(t);

	p= *pp;
	ASN1_put_object(&p,1,tot,V_ASN1_SEQUENCE,V_ASN1_UNIVERSAL);

	i2d_ASN1_INTEGER(&bs,&p);

	bs.data=(unsigned char *)Malloc(max+4);
	if (bs.data == NULL)
		{
		ASN1err(ASN1_F_I2D_RSAPRIVATEKEY,ERR_R_MALLOC_FAILURE);
		return(-1);
		}

	for (i=1; i<d->k+7; i++)
		{
		if(num[i]->neg) {
		bs.length=BN_bn2bin(num[i],bs.data);
		bs.type=V_ASN1_NEG_INTEGER;
		}
		else {
		bs.length=BN_bn2bin(num[i],bs.data);
		bs.type=V_ASN1_INTEGER;
		}
		i2d_ASN1_INTEGER(&bs,&p);
		}
	BN_clear_free(num[3]);
	BN_clear_free(num[4]);
	BN_clear_free(num[6]);
	Free((char *)bs.data);
	*pp=p;
	return(t);
	}

RSA_SHARE *
PEM_read_RSA_share(fp, x, cb, u)
     FILE * fp;
     RSA_SHARE ** x;
     int (*cb)();
     void * u;
{
  return((RSA_SHARE *)PEM_ASN1_read((char *(*)())d2i_RSAShare,
				    PEM_STRING_RSA_SHARE, fp, (char **) x, cb, u));
}

RSA_SHARE *
PEM_read_bio_RSA_share(bp, x, cb, u)
     BIO * bp;
     RSA_SHARE ** x;
     int (*cb)();
     void * u;
{
  return((RSA_SHARE *)PEM_ASN1_read_bio((char *(*)())d2i_RSAShare,
					PEM_STRING_RSA_SHARE, bp, (char **) x,
					cb, u));
}

int
PEM_write_RSA_share(fp, x, enc, kstr, klen, cb, u)
     FILE * fp;
     RSA_SHARE * x;
     EVP_CIPHER * enc;
     unsigned char * kstr;
     int klen;
     int (*cb)();
     void * u;
{
  return PEM_ASN1_write((int (*)()) i2d_RSAShare, PEM_STRING_RSA_SHARE, fp,
			(char *) x, enc, kstr, klen, cb, u);
}

int PEM_write_bio_RSA_share(bp, x, enc, kstr, klen, cb, u)
     BIO * bp;
     RSA_SHARE * x;
     EVP_CIPHER * enc;
     unsigned char * kstr;
     int klen;
     int (*cb)();
     void * u;
{
  return PEM_ASN1_write_bio((int (*)()) i2d_RSAShare, PEM_STRING_RSA_SHARE, bp,
			    (char *) x, enc, kstr, klen, cb, u);
}

RSA_SHARED_KEY *
PEM_read_RSA_shared_key(fp, x, cb, u)
     FILE * fp;
     RSA_SHARED_KEY ** x;
     int (*cb)();
     void * u;
{
  return((RSA_SHARED_KEY *)PEM_ASN1_read((char *(*)())d2i_RSASharedKey,
					 PEM_STRING_RSA_SHARED_KEY, fp,
					 (char **) x, cb, u));
}

RSA_SHARED_KEY *
PEM_read_bio_RSA_shared_key(bp, x, cb, u)
     BIO * bp;
     RSA_SHARED_KEY ** x;
     int (*cb)();
     void * u;
{
  return((RSA_SHARED_KEY *)PEM_ASN1_read_bio((char *(*)())d2i_RSASharedKey,
					     PEM_STRING_RSA_SHARED_KEY, bp,
					     (char **) x, cb, u));
}

int
PEM_write_RSA_shared_key(fp, x, enc, kstr, klen, cb, u)
     FILE * fp;
     RSA_SHARED_KEY * x;
     EVP_CIPHER * enc;
     unsigned char * kstr;
     int klen;
     int (*cb)();
     void * u;
{
  return PEM_ASN1_write((int (*)()) i2d_RSASharedKey,
			PEM_STRING_RSA_SHARED_KEY, fp,
			(char *) x, enc, kstr, klen, cb, u);
}

int PEM_write_bio_RSA_shared_key(bp, x, enc, kstr, klen, cb, u)
     BIO * bp;
     RSA_SHARED_KEY * x;
     EVP_CIPHER * enc;
     unsigned char * kstr;
     int klen;
     int (*cb)();
     void * u;
{
  return PEM_ASN1_write_bio((int (*)()) i2d_RSASharedKey,
			    PEM_STRING_RSA_SHARED_KEY, bp,
			    (char *) x, enc, kstr, klen, cb, u);
}
