/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

/* Private RSA share handlers */

#include <stdlib.h>

#include "KEY.h"

static int
KEY_priv_ex_data_new(obj, item, ed, index, argl, argp)
     RSA_SHARE * obj;
     char * item;
     CRYPTO_EX_DATA * ed;
     int index;
     long argl;
     char * argp;
{
  int i;
  RSA_SHARE_APP_DATA * data;

  /* TODO: only do this if we know the (RSA *) is really an (RSA_SHARE *) */

  if((data = malloc(sizeof(RSA_SHARE_APP_DATA))) == NULL)
    return 0;

  data->ver = 1;
  data->k = 0;
  for(i = 0; i < MAX_RSA_SHARING; ++i)
    data->d[i] = NULL;

  CRYPTO_set_ex_data(ed, index, (char *) data);
  return 1;
}

static void
KEY_priv_ex_data_free(obj, item, ed, index, argl, argp)
     RSA_SHARE * obj;
     char * item;
     CRYPTO_EX_DATA * ed;
     int index;
     long argl;
     char * argp;
{
  if(item)
    free(item);
}

static int
KEY_priv_ex_data_dup(obj_to, obj_from, newptr, index, argl, argp)
     char * obj_to;
     char * obj_from;
     char ** newptr;
     int index;
     long argl;
     char * argp;
{
  /* TODO */
  return 0;
}

static int ex_idx;

void
KEY_RSA_share_initialize()
{
  ex_idx = RSA_get_ex_new_index(0, NULL, KEY_priv_ex_data_new,
				KEY_priv_ex_data_dup, KEY_priv_ex_data_free);
}

PACKAGE RSA_SHARE_APP_DATA *
KEY_RSA_share_get_data(rsa)
     RSA_SHARE * rsa;
{
  return (RSA_SHARE_APP_DATA *) RSA_get_ex_data(rsa, ex_idx);
}

static int
KEY_priv_init(rsa)
     RSA * rsa;
{
  /*
  int i;
  RSA_SHARE_APP_DATA * data;

  if((data = malloc(sizeof(RSA_SHARE_APP_DATA))) == NULL)
    return 0;

  RSA_set_app_data(rsa, (char *) data);

  data->ver = 1;
  data->k = 0;
  for(i = 0; i < MAX_RSA_SHARING; ++i)
    data->d[i] = NULL;
  */

  return 1;
}

static int
KEY_priv_finish(rsa)
     RSA * rsa;
{
  /*
  if(RSA_get_app_data(rsa))
    free(RSA_get_app_data(rsa));
  */

  return 1;
}

/* The private RSA share method passes most operations along to
   the underlying method */

static RSA_METHOD rsa_private_share_meth = {
  "Tom Wu's Private Share RSA / EAY PKCS#1",
  NULL,		/* Inherit from underlying SSLeay method */
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  KEY_priv_init,
  KEY_priv_finish
};

RSA_METHOD *
KEY_RSA_share_meth_ssleay()
{
  RSA_METHOD * meth;

  if(rsa_private_share_meth.rsa_pub_enc == NULL) {
    meth = RSA_PKCS1_SSLeay();	/* Fill in fields from underlying meth */
    rsa_private_share_meth.rsa_pub_enc = meth->rsa_pub_enc;
    rsa_private_share_meth.rsa_pub_dec = meth->rsa_pub_dec;
    rsa_private_share_meth.rsa_priv_enc = meth->rsa_priv_enc;
    rsa_private_share_meth.rsa_priv_dec = meth->rsa_priv_dec;
    rsa_private_share_meth.rsa_mod_exp = meth->rsa_mod_exp;
    rsa_private_share_meth.bn_mod_exp = meth->bn_mod_exp;
  }
  return &rsa_private_share_meth;
}

RSA_SHARE *
KEY_RSA_share_new()
{
  return (RSA_SHARE *) RSA_new_method(KEY_RSA_share_meth_ssleay());
}

RSA_SHARE *
KEY_RSA_share_create(n, e)
     BIGNUM * n;
     BIGNUM * e;
{
  RSA_SHARE * rsa;

  if((rsa = KEY_RSA_share_new()) == NULL)
    return NULL;

  rsa->n = BN_dup(n);
  rsa->e = BN_dup(e);

  return rsa;
}

void
KEY_RSA_share_free(share)
     RSA_SHARE * share;
{
  RSA_free(share);
}

void
KEY_RSA_share_set_threshold_params(k, np, t)
     RSA_SHARE * k;
     int np, t;
{
  RSA_SHARE_APP_DATA * data;

  if((data = KEY_RSA_share_get_data(k)) == NULL)
    return;
  data->nparties = np;
  data->threshold = t;
}

void
KEY_RSA_share_set_version(k, v)
     RSA_SHARE * k;
     int v;
{
  RSA_SHARE_APP_DATA * data;

  if((data = KEY_RSA_share_get_data(k)) == NULL)
    return;
  data->ver = v;
}

void
KEY_RSA_share_add_d(k, d)
     RSA_SHARE * k;
     BIGNUM * d;
{
  RSA_SHARE_APP_DATA * data;

  if((data = KEY_RSA_share_get_data(k)) == NULL)
    return;

  if(data->k >= MAX_RSA_SHARING)
    return;

  data->d[data->k++] = BN_dup(d);
}


RSA_SHARE *
KEY_RSA_share_read(file)
     char * file;
{
  RSA_SHARE * share;
  FILE * fp;

  if((fp = fopen(file, "r")) == NULL)
    return NULL;

  if(KEY_use_callback())
    share = PEM_read_RSA_share(fp, NULL, KEY_pp_callback, NULL);
  else
    share = PEM_read_RSA_share(fp, NULL, NULL, NULL);

  fclose(fp);
  return share;
}

int
KEY_RSA_share_write(share, file, enc_alg, key, keylen)
     RSA_SHARE * share;
     const char * file;
     const char * enc_alg;
     unsigned char * key;
     int keylen;
{
  EVP_CIPHER * cipher = NULL;

  if(enc_alg == NULL) {
    enc_alg = KEY_get_cipher_name();
    if(strlen(enc_alg) == 0)
      enc_alg = NULL;
  }
  if(enc_alg) {
    cipher = (EVP_CIPHER *)EVP_get_cipherbyname(enc_alg);
    if(cipher == NULL)
      return -1;
  }
  return KEY_RSA_share_write_cipher(share, file, cipher, key, keylen);
}

int
KEY_RSA_share_write_cipher(share, file, cipher, key, keylen)
     RSA_SHARE * share;
     const char * file;
     EVP_CIPHER * cipher;
     unsigned char * key;
     int keylen;
{
  FILE * fp;

  if((fp = fopen(file, "w")) == NULL)
    return -1;

  PEM_write_RSA_share(fp, share, cipher, key, keylen,
		      KEY_use_callback() ? KEY_pp_callback : NULL, NULL);
  fclose(fp);

  return 0;
}

BIGNUM *
KEY_RSA_share_n(share)
     RSA_SHARE * share;
{
  return share->n;
}

BIGNUM *
KEY_RSA_share_e(share)
     RSA_SHARE * share;
{
  return share->e;
}

int
KEY_RSA_share_nparties(share)
     RSA_SHARE * share;
{
  RSA_SHARE_APP_DATA * data;

  if(share == NULL || (data = KEY_RSA_share_get_data(share)) == NULL)
    return -1;

  return data->nparties;
}

int
KEY_RSA_share_threshold(share)
     RSA_SHARE * share;
{
  RSA_SHARE_APP_DATA * data;

  if(share == NULL || (data = KEY_RSA_share_get_data(share)) == NULL)
    return -1;

  return data->threshold;
}

int
KEY_RSA_share_version(share)
     RSA_SHARE * share;
{
  RSA_SHARE_APP_DATA * data;

  if(share == NULL || (data = KEY_RSA_share_get_data(share)) == NULL)
    return -1;

  return data->ver;
}

int
KEY_RSA_share_k(share)
     RSA_SHARE * share;
{
  RSA_SHARE_APP_DATA * data;

  if(share == NULL || (data = KEY_RSA_share_get_data(share)) == NULL)
    return -1;

  return data->k;
}

BIGNUM *
KEY_RSA_share_d(share, num)
     RSA_SHARE * share;
     int num;
{
  RSA_SHARE_APP_DATA * data;

  if(share == NULL || (data = KEY_RSA_share_get_data(share)) == NULL)
    return NULL;

  return data->d[num];
}

/* Mike's function */
void KEY_RSA_share_print_all_d(RSA_SHARE *share) {

  RSA_SHARE_APP_DATA * data;
  char *txt = NULL;
  int i;

  if(share == NULL || (data = KEY_RSA_share_get_data(share)) == NULL)
    return;

  for (i=0; i<data->k; i++) {
    txt = BN_bn2hex(data->d[i]);
    if (txt) {
      UTL_Log("KEY_RSA_share_print_all_d", LOG_NOTIFY, "d_%d = %s", i, txt);
      free(txt);
      txt = NULL;
    }
  }
}
