/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

/* RSA shared key handlers */

#include <stdlib.h>

#include "KEY.h"
#include "COM.h"
#include "UTL.h"
#include "TLB.h"

static int
KEY_priv_ex_data_new(obj, item, ed, index, argl, argp)
     RSA_SHARED_KEY * obj;
     char * item;
     CRYPTO_EX_DATA * ed;
     int index;
     long argl;
     char * argp;
{
  int i;
  RSA_SHARED_KEY_APP_DATA * data;

  /* TODO: only do this if we know the (RSA *) is really an (RSA_SHARED_KEY *) */

  if((data = malloc(sizeof(RSA_SHARED_KEY_APP_DATA))) == NULL)
    return 0;

  data->k = 0;
  data->g = NULL;
  for(i = 0; i < MAX_RSA_SHARING; ++i)
    data->gd[i] = NULL;
  data->ctx = NULL;

  CRYPTO_set_ex_data(ed, index, (char *) data);
  return 1;
}

static void
KEY_priv_ex_data_free(obj, item, ed, index, argl, argp)
     RSA_SHARED_KEY * obj;
     char * item;
     CRYPTO_EX_DATA * ed;
     int index;
     long argl;
     char * argp;
{
  RSA_SHARED_KEY_APP_DATA * data = (RSA_SHARED_KEY_APP_DATA *) item;

  if(data != NULL) {
    if(data->ctx)
      TLB_CTX_free(data->ctx);
    free(data);
  }
}

static int
KEY_priv_ex_data_dup(obj_to, obj_from, newptr, index, argl, argp)
     char * obj_to;
     char * obj_from;
     char ** newptr;
     int index;
     long argl;
     char * argp;
{
  /* TODO */
  return 0;
}

static int ex_idx;

void
KEY_RSA_shared_key_initialize()
{
  ex_idx = RSA_get_ex_new_index(0, NULL, KEY_priv_ex_data_new,
				KEY_priv_ex_data_dup, KEY_priv_ex_data_free);
}

PACKAGE RSA_SHARED_KEY_APP_DATA *
KEY_RSA_shared_key_get_data(rsa)
     RSA_SHARED_KEY * rsa;
{
  return (RSA_SHARED_KEY_APP_DATA *) RSA_get_ex_data(rsa, ex_idx);
}

static int
KEY_priv_init(rsa)
     RSA * rsa;
{
  /*
  int i;
  RSA_SHARED_KEY_APP_DATA * data;

  if((data = malloc(sizeof(RSA_SHARED_KEY_APP_DATA))) == NULL)
    return 0;

  RSA_set_app_data(rsa, (char *) data);

  data->k = 0;
  data->g = NULL;
  for(i = 0; i < MAX_RSA_SHARING; ++i)
    data->gd[i] = NULL;
  data->ctx = NULL;
  */

  return 1;
}

static int
KEY_priv_finish(rsa)
     RSA * rsa;
{
  /*
  RSA_SHARED_KEY_APP_DATA * data;

  if((data = (RSA_SHARED_KEY_APP_DATA *) RSA_get_app_data(rsa)) != NULL) {
    if(data->ctx)
      TLB_CTX_free(data->ctx);
    free(data);
  }
  */

  return 1;
}

/* The RSA shared key method passes some operations along to
   the underlying method.  The remaining operations (private
   operations) are handled by TLB. */

static RSA_METHOD rsa_shared_key_meth = {
  "Tom Wu's Shared Key RSA / EAY PKCS#1",
  NULL,		/* Inherit from underlying SSLeay method */
  NULL,
  TLB_private_encrypt,
  TLB_private_decrypt,
  TLB_DecryptWithKey,
  NULL,
  KEY_priv_init,
  KEY_priv_finish
};

RSA_METHOD *
KEY_RSA_shared_key_meth_ssleay()
{
  RSA_METHOD * meth;

  if(rsa_shared_key_meth.rsa_pub_enc == NULL) {
    meth = RSA_PKCS1_SSLeay();	/* Fill in fields from underlying meth */
    rsa_shared_key_meth.rsa_pub_enc = meth->rsa_pub_enc;
    rsa_shared_key_meth.rsa_pub_dec = meth->rsa_pub_dec;
    /*
    rsa_shared_key_meth.rsa_priv_enc = meth->rsa_priv_enc;
    rsa_shared_key_meth.rsa_priv_dec = meth->rsa_priv_dec;
    rsa_shared_key_meth.rsa_mod_exp = meth->rsa_mod_exp;
    */
    rsa_shared_key_meth.bn_mod_exp = meth->bn_mod_exp;
  }
  return &rsa_shared_key_meth;
}

RSA_SHARED_KEY *
KEY_RSA_shared_key_new()
{
  return (RSA_SHARE *) RSA_new_method(KEY_RSA_shared_key_meth_ssleay());
}

RSA_SHARED_KEY *
KEY_RSA_shared_key_create(n, e)
     BIGNUM * n;
     BIGNUM * e;
{
  RSA_SHARE * rsa;

  if((rsa = KEY_RSA_shared_key_new()) == NULL)
    return NULL;

  rsa->n = BN_dup(n);
  rsa->e = BN_dup(e);

  return rsa;
}

void
KEY_RSA_shared_key_free(key)
     RSA_SHARED_KEY * key;
{
  RSA_free(key);
}

void
KEY_RSA_shared_key_set_threshold_params(k, np, t)
     RSA_SHARED_KEY * k;
     int np, t;
{
  RSA_SHARED_KEY_APP_DATA * data;

  if((data = KEY_RSA_shared_key_get_data(k)) == NULL)
    return;

  data->nparties = np;
  data->threshold = t;
}

void
KEY_RSA_shared_key_set_ctx(k, ctx)
     RSA_SHARED_KEY * k;
     TLB_CTX * ctx;
{
  RSA_SHARED_KEY_APP_DATA * data;

  if((data = KEY_RSA_shared_key_get_data(k)) == NULL)
    return;

  data->ctx = ctx;
}

void
KEY_RSA_shared_key_set_g(k, g)
     RSA_SHARED_KEY * k;
     BIGNUM * g;
{
  RSA_SHARED_KEY_APP_DATA * data;

  if((data = KEY_RSA_shared_key_get_data(k)) == NULL)
    return;

  data->g = BN_dup(g);
}

void
KEY_RSA_shared_key_add_gd(k, gd)
     RSA_SHARED_KEY * k;
     BIGNUM * gd;
{
  RSA_SHARED_KEY_APP_DATA * data;

  if((data = KEY_RSA_shared_key_get_data(k)) == NULL)
    return;

  if(data->k >= MAX_RSA_SHARING)	/* TJW: This test may be inaccurate */
    return;

  data->gd[data->k++] = BN_dup(gd);
}

RSA_SHARED_KEY *
KEY_RSA_shared_key_read(file)
     char * file;
{
  RSA_SHARED_KEY * key;
  FILE * fp;

  if((fp = fopen(file, "r")) == NULL)
    return NULL;

  key = PEM_read_RSA_shared_key(fp, NULL, NULL, NULL);

  fclose(fp);
  return key;
}

int
KEY_RSA_shared_key_write(key, file)
     RSA_SHARED_KEY * key;
     char * file;
{
  FILE * fp;

  if((fp = fopen(file, "w")) == NULL)
    return -1;

  PEM_write_RSA_shared_key(fp, key, NULL, NULL, 0, NULL, NULL);
  fclose(fp);

  return 0;
}

BIGNUM *
KEY_RSA_shared_key_n(key)
     RSA_SHARED_KEY * key;
{
  return key->n;
}

BIGNUM *
KEY_RSA_shared_key_e(key)
     RSA_SHARED_KEY * key;
{
  return key->e;
}

int
KEY_RSA_shared_key_nparties(key)
     RSA_SHARED_KEY * key;
{
  RSA_SHARED_KEY_APP_DATA * data;

  if(key == NULL || (data = KEY_RSA_shared_key_get_data(key)) == NULL)
    return -1;

  return data->nparties;
}

int
KEY_RSA_shared_key_threshold(key)
     RSA_SHARED_KEY * key;
{
  RSA_SHARED_KEY_APP_DATA * data;

  if(key == NULL || (data = KEY_RSA_shared_key_get_data(key)) == NULL)
    return -1;

  return data->threshold;
}

BIGNUM *
KEY_RSA_shared_key_g(key)
     RSA_SHARED_KEY * key;
{
  RSA_SHARED_KEY_APP_DATA * data;

  if(key == NULL || (data = KEY_RSA_shared_key_get_data(key)) == NULL)
    return NULL;

  return data->g;
}

int
KEY_RSA_shared_key_k(key)
     RSA_SHARED_KEY * key;
{
  RSA_SHARED_KEY_APP_DATA * data;

  if(key == NULL || (data = KEY_RSA_shared_key_get_data(key)) == NULL)
    return -1;

  return data->k;
}

TLB_CTX *
KEY_RSA_shared_key_ctx(key)
     RSA_SHARED_KEY * key;
{
  RSA_SHARED_KEY_APP_DATA * data;

  if(key == NULL || (data = KEY_RSA_shared_key_get_data(key)) == NULL)
    return NULL;

  return data->ctx;
}

BIGNUM *
KEY_RSA_shared_key_gd(key, num)
     RSA_SHARED_KEY * key;
     int num;
{
  RSA_SHARED_KEY_APP_DATA * data;

  if(key == NULL || (data = KEY_RSA_shared_key_get_data(key)) == NULL)
    return NULL;

  return data->gd[num];
}
