/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

#include "ITTC.h"

/* NOTE: Assumes BN_ULONG is four bytes */

static BN_ULONG
block_xor(n)
     BIGNUM * n;
{
  int b, i, offset, w;
  BN_ULONG r;
  BN_ULONG * blks;
  unsigned char * a;

  b = BN_num_bytes(n);
  if(b & 3)
    offset = 4 - (b & 3);
  else
    offset = 0;
  w = (b + 3) / 4;
  if(w == 0 || (blks = (BN_ULONG *) malloc(w * sizeof(BN_ULONG))) == NULL)
    return 0;
  blks[0] = 0;
  BN_bn2bin(n, offset + (unsigned char *) blks);
  r = 0;
  for(i = 0; i < w; ++i)
    r ^= blks[i];
  free(blks);
  a = (unsigned char *) &r;	/* Big-endian systems can return r directly */
  return (a[0] << 24) | (a[1] << 16) | (a[2] << 8) | a[3];
}

BN_ULONG
KEY_RSA_get_ID(rsa)
     RSA * rsa;
{
  return block_xor(rsa->n) ^ block_xor(rsa->e);
}

/* EVP-handling code */

static char * k_passphrase = NULL;
static char k_cipher[64] = "des";		// Default setting

void
KEY_set_passphrase(pp)
     const char * pp;
{
  if(pp)
    k_passphrase = strdup(pp);
}

void
KEY_set_cipher_name(cname)
     const char * cname;
{
  strcpy(k_cipher, cname);
}

char *
KEY_get_cipher_name()
{
  return k_cipher;
}

int
KEY_use_callback()
{
  return (k_passphrase != NULL);
}

int
KEY_pp_callback(buf, sz, is_write)
     char * buf;
     int sz, is_write;
{
  if(KEY_use_callback()) {
    strncpy(buf, k_passphrase, sz);
    return strlen(k_passphrase);
  }
  else
    return 0;
}
