/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

/*------------------------------------------------------------------------*/
/*--									--*/
/*--	TLB_CheckSig.c							--*/
/*--									--*/
/*--	Check a signature      						--*/
/*--									--*/
/*------------------------------------------------------------------------*/

#include "ITTC.h"

int
TLB_CheckSig(BIGNUM * sig, BIGNUM ** sigs, BIGNUM * msg, RSA * pubkey,
		 long coalition, TLB_INTERNAL_CTX * ctx)
{
  int i;
  BIGNUM *check = NULL, *t = NULL;
  /*char *tmp;*/
  int heapState;

  /* Initialize */
  heapState = UTL_GetHeapState(ctx->heap);
  check = UTL_GetBN(ctx->heap);
  if (check == NULL) goto error;
  t     = UTL_GetBN(ctx->heap);
  if (t == NULL) goto error;

  /* Multiply together all shares of signature */
  if (!BN_set_word(sig, 1)) goto error;
  for(i=0; i < ctx->numServers; i++) {

    if ( (sigs[i]) && (!BN_is_zero(sigs[i])) ) {
      /*
      tmp = BN_bn2hex(sigs[i]);
      if (tmp) {
	UTL_Log("TLB_CheckSig", LOG_TEST, "Contribution from server %d: %s", i, tmp);
	free(tmp);
      }
      */
      if (!BN_mod_mul(sig, sig, sigs[i], pubkey->n, ctx->BNctx)) goto error;
    } 
  }

  /* Encrypt the signature */
  if (!BN_mod_exp(check, sig, pubkey->e, pubkey->n, ctx->BNctx)) goto error;

  /* Check if the signature was correct, then return */
  i = BN_cmp(check, msg);

  if(i == 0) {
    UTL_Log("TLB_CheckSig", LOG_INFO, "Signature matches");
    UTL_RevertHeap(heapState, ctx->heap);
    return RC_OK;
  }

  /* Check if minus the signature was correct, then return */
  if (!BN_sub(t, pubkey->n, check)) goto error;
  i = BN_cmp(check, msg);

  if(i == 0) {
    UTL_Log("TLB_CheckSig", LOG_INFO, "Minus signature matches");
    if (!BN_sub(t, pubkey->n, sig)) goto error;
    if (!BN_copy(sig, t)) goto error;
    UTL_RevertHeap(heapState, ctx->heap);
    return RC_OK;
  }

  UTL_RevertHeap(heapState, ctx->heap);
  UTL_Log("TLB_CheckSig", LOG_WARNING, "Signature does not match");
  return TLB_CORRUPT;


 error:
  UTL_RevertHeap(heapState, ctx->heap);
  UTL_Log("TLB_CheckSig", LOG_INFO, "Error");
  return RC_ERROR;
}

