/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

/*------------------------------------------------------------------------*/
/*--									--*/
/*--	TLB_Request.c							--*/
/*--									--*/
/*--	Send a request to the share server.				--*/
/*------------------------------------------------------------------------*/

#include "ITTC.h"

int TLB_SendRequest(COM_STREAM *s, RSA_SHARED_KEY *pubkey, 
		     int op, BIGNUM *msg, long int coalition, 
		     long int offline, long int corrupt, TLB_INTERNAL_CTX *ctx)
{
  
  /*--------------------------------------------------------------------*/
  /*	Message format:							*/
  /*	  < op, msg , coalition, keyID, offline, corrupt >		*/
  /*    op == TLB_OP_APPLY_DECRYPT: return decryption			*/
  /*    op == TLB_OP_APPLY_SIG: return signature			*/
  /*    op == TLB_OP_APPLY_HASHED: return hashed signature		*/
  /*--------------------------------------------------------------------*/

  if (COM_write_string(s, "COMMAND") < 0) {
    UTL_Log("TLB_SendRequest", LOG_FATAL, "Error sending header.");
    return RC_ERROR;
  }
 
  if (COM_write_int(s, op) < 0) {
    UTL_Log("TLB_SendRequest", LOG_FATAL, "Error sending opcode.");
    return RC_ERROR;
  }
 
  if (COM_write_bn(s, msg) < 0) {
    UTL_Log("TLB_SendRequest", LOG_FATAL, "Error sending message.");
    return RC_ERROR;
  }
  
  if (COM_write_int(s, coalition) < 0) {
    UTL_Log("TLB_SendRequest", LOG_FATAL, "Error sending coalition.");
    return RC_ERROR;
  }
  
  if (COM_write_int(s, KEY_RSA_get_ID(pubkey)) < 0) {
    UTL_Log("TLB_SendRequest", LOG_FATAL, "Error sending key ID.");
    return RC_ERROR;
  }
  
  if (COM_write_int(s, corrupt) < 0) {
    UTL_Log("TLB_SendRequest", LOG_FATAL, "Error sending corrupt.");
    return RC_ERROR;
  }
  
  if (COM_write_int(s, offline) < 0) {
    UTL_Log("TLB_SendRequest", LOG_FATAL, "Error sending offline.");
    return RC_ERROR;
  }

  return RC_OK;

}
