/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

/*----------------------------------------------------------*/
/*--                                                      --*/
/*--  UTL_Check.c                                         --*/
/*--                                                      --*/
/*--  Functions to check the responses of servers         --*/
/*--                                                      --*/
/*----------------------------------------------------------*/

#include "ITTC.h"
int UTL_CheckResponse(COM_STREAM *stream, void *data, int flag)
{
  int rc;
  COM_STREAM *tStream = NULL;
  void *vt = NULL;
  char dataS[256];
  int dataI;
  int *temp = NULL;

  rc = COM_select_v(stream->ctx, &vt, &stream, 1, 
		     COM_SELECT_READ | COM_SELECT_TIMEOUT | COM_SELECT_CLOSE);

  if (rc == COM_SELECTED_TIMEOUT) {
    UTL_Log("UTL_CheckResponse", LOG_NOTIFY, "Timeout.");
    return RC_TIMEOUT;
  }

  if (rc == COM_SELECTED_CLOSE) {
    UTL_Log("UTL_CheckResponse", LOG_NOTIFY, "Stream closed.");
    return RC_CLOSE;
  }

  if (rc != COM_SELECTED_READ) {
    UTL_Log("UTL_CheckResponse", LOG_INFO, "Select returned unexpected value %d.", rc);
    return RC_ERROR;
  }

  tStream = (COM_STREAM*)vt;

  /* Read the response from the stream */
  switch (flag) {

    /* This is a string input */
    case UTL_TYPE_STRING:
      if (COM_read_string(tStream, dataS) < 0) {
	UTL_Log("UTL_CheckResponse", LOG_INFO, "Error reading response string");
	return RC_ERROR;
      }
      if (strcmp(dataS, (char*)data) != 0) {
	UTL_Log("UTL_CheckResponse", LOG_INFO, "ERROR: Server said \"%s\".", dataS);
	return RC_ERROR;
      }
      break;

    /* This is an integer input */
    case UTL_TYPE_INT:
      if (COM_read_int(tStream, &dataI) < 0) {
	UTL_Log("UTL_CheckResponse", LOG_INFO, "Error reading response integer");
	return RC_ERROR;
      }
      temp = (int*)data;
      if ( dataI != *temp ) {
	UTL_Log("UTL_CheckResponse", LOG_INFO, "ERROR: Server said [%d].", dataI);
	return RC_ERROR;
      }
      break;

    /* Unknown type */
    default:
      UTL_Log("UTL_CheckResponse", LOG_INFO, "Unknown type.");
      return RC_ERROR;
  }
  
  return RC_OK;
}



int UTL_CheckResponseStr(COM_STREAM *stream, char *msg) {
  return UTL_CheckResponse(stream, (void*)msg, UTL_TYPE_STRING);
}

int UTL_CheckResponseInt(COM_STREAM *stream, int goal) {
  return UTL_CheckResponse(stream, (void*)&goal, UTL_TYPE_INT);
}
