/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

/* UTL_Time - platform-independent time abstraction */

#include "ITTC.h"
#include "UTL_Time.h"

#ifdef WIN32

int
UTL_TimeGetCurrent(t)
	UTL_TIME * t;
{
	GetSystemTimeAsFileTime(&t->ft);
	return RC_OK;
}

long
UTL_TimeSubtractMsec(t2, t1)
	UTL_TIME * t2;
	UTL_TIME * t1;
{
	return MulDiv(t2->ft.dwHighDateTime - t1->ft.dwHighDateTime, 0x10000000, 625) +
		((long) t2->ft.dwLowDateTime - t1->ft.dwLowDateTime) / 10000L;
}

int
UTL_TimeToString(t, buf)
	UTL_TIME * t;
	char * buf;
{
	SYSTEMTIME st;

	FileTimeToSystemTime(&t->ft, &st);
	sprintf(buf, "%d-%d-%d %02d:%02d:%02d",
		st.wMonth, st.wDay, st.wYear, st.wHour, st.wMinute, st.wSecond);
	return RC_OK;
}

int
UTL_TimeToHMSMDY(t, buf)
     UTL_TIME * t;
     char buf[HMSMDY_SIZE + 1];
{
	SYSTEMTIME st;

	FileTimeToSystemTime(&t->ft, &st);
	sprintf(buf, "%02d%02d%02d%02d%02d%04d", st.wHour, st.wMinute,
		st.wSecond, st.wMonth, st.wDay, 1900 + st.wYear);
	return RC_OK;
}

#else

#include <time.h>
#include <sys/time.h>

int
UTL_TimeGetCurrent(t)
	UTL_TIME * t;
{
	gettimeofday(&t->tm, NULL);
	return RC_OK;
}

long
UTL_TimeSubtractMsec(t2, t1)
	UTL_TIME * t2;
	UTL_TIME * t1;
{
	return 1000L * (t2->tm.tv_sec - t1->tm.tv_sec) + (t2->tm.tv_usec - t1->tm.tv_usec) / 1000L;
}

int
UTL_TimeToString(t, buf)
	UTL_TIME * t;
	char * buf;
{
	struct tm gm;

	strftime(buf, UTL_TIME_STRING_LENGTH, "%m-%d-%Y %T", gmtime_r(&t->tm.tv_sec, &gm));
	return RC_OK;
}

int
UTL_TimeToHMSMDY(t, buf)
	UTL_TIME * t;
	char buf[HMSMDY_SIZE + 1];
{
	struct tm gm;

	strftime(buf, UTL_TIME_STRING_LENGTH, "%H%M%S%m%d%Y", gmtime_r(&t->tm.tv_sec, &gm));
	return RC_OK;
}

#endif

