/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

#ifdef STHREADS
#include <stdlib.h>
#include <thread.h>
#include "RC.h"
#include "UTL_cond.h"

/* Solaris threads-based condition variables */

static int
UTL_cond_sthread_init(cond)
     UTL_COND * cond;
{
  if((cond->data = malloc(sizeof(cond_t))) == NULL)
    return RC_ERROR;
  if(cond_init((cond_t *)cond->data, 0, NULL) != 0)
    return RC_ERROR;
  return RC_OK;
}

/* We assume that the PTHREADS mutex has the pthread_mutex_t
 * directly in the data field. */

static void
UTL_cond_sthread_wait(cond, mutex)
     UTL_COND * cond;
     UTL_MUTEX * mutex;
{
  cond_wait((cond_t *)cond->data, (mutex_t *)mutex->data);
}

static int
UTL_cond_sthread_wait_timed(cond, mutex, ms)
     UTL_COND * cond;
     UTL_MUTEX * mutex;
     int ms;
{
  struct timespec atm;
  struct timeval tm;

  gettimeofday(&tm, NULL);
  atm.tv_sec = tm.tv_sec + (ms / 1000);
  atm.tv_nsec = 1000 * (tm.tv_usec + 1000 * (ms % 1000));
  if(atm.tv_nsec > 1000000000) {
    atm.tv_nsec -= 1000000000;
    atm.tv_sec++;
  }
  if(cond_timedwait((cond_t *)cond->data, (mutex_t *)mutex->data, &atm) != 0)
    return RC_TIMEOUT;
  return RC_OK;
}

static void
UTL_cond_sthread_notify(cond)
     UTL_COND * cond;
{
  cond_signal((cond_t *)cond->data);
}

static void
UTL_cond_sthread_notify_all(cond)
     UTL_COND * cond;
{
  cond_broadcast((cond_t *)cond->data);
}

static void
UTL_cond_sthread_destroy(cond)
     UTL_COND * cond;
{
  cond_destroy((cond_t *)cond->data);
  free(cond->data);
}

static UTL_COND_METH sthread_method = {
  UTL_cond_sthread_init,
  UTL_cond_sthread_wait,
  UTL_cond_sthread_wait_timed,
  UTL_cond_sthread_notify,
  UTL_cond_sthread_notify_all,
  UTL_cond_sthread_destroy
};

UTL_COND_METH *
UTL_cond_sthread_method()
{
  return &sthread_method;
}
#endif /* STHREADS */
