/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */
#ifdef WIN32

/* TJW: Microsoft Brain Damage - we have to tell it which version it is! */

#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0400
#endif

#include <windows.h>
#include "RC.h"
#include "UTL_cond.h"

/*
WINBASEAPI
DWORD
WINAPI
SignalObjectAndWait(
    HANDLE hObjectToSignal,
    HANDLE hObjectToWaitOn,
    DWORD dwMilliseconds,
    BOOL bAlertable
    );
	*/

/* win32-based condition variables */

static int
UTL_cond_win32_init(cond)
     UTL_COND * cond;
{
	/*
  if((cond->data = (void *) malloc(2 * sizeof(HANDLE))) == NULL)
    return RC_ERROR;
  if((((HANDLE *)cond->data)[0] = CreateEvent(NULL, FALSE, FALSE, "UTL_cond_win32_single")) == NULL ||
	  (((HANDLE *)cond->data)[1] = CreateEvent(NULL, TRUE, FALSE, "UTL_cond_win32_all")) == NULL)
	  return RC_ERROR;
	  */
  if((cond->data = CreateEvent(NULL, TRUE, FALSE, NULL)) == NULL)
	  return RC_ERROR;
  return RC_OK;
}

/* TODO/TRY: use SetEvent w/an auto-reset event, and do ResetEvent before each wait */

static void
UTL_cond_win32_wait(cond, mutex)
     UTL_COND * cond;
     UTL_MUTEX * mutex;
{
  /* TODO: make these atomic under WIN32 */
	/*
  UTL_mutex_unlock(mutex);
  WaitForMultipleObjects(2, (HANDLE *)cond->data, FALSE, INFINITE);
  */
	SignalObjectAndWait((HANDLE)mutex->data, (HANDLE)cond->data, INFINITE, FALSE);
  UTL_mutex_lock(mutex);
}

static int
UTL_cond_win32_wait_timed(cond, mutex, ms)
     UTL_COND * cond;
     UTL_MUTEX * mutex;
     int ms;
{
  /* TODO: make these atomic under WIN32 */
	/*
  UTL_mutex_unlock(mutex);
  if(WaitForMultipleObjects(2, (HANDLE *)cond->data, FALSE, ms) == WAIT_TIMEOUT) {
	  UTL_mutex_lock(mutex);
    return RC_TIMEOUT;
  }
  */
	if(SignalObjectAndWait((HANDLE)mutex->data, (HANDLE)cond->data, ms, FALSE) == WAIT_TIMEOUT) {
		UTL_mutex_lock(mutex);
		return RC_TIMEOUT;
	}
  UTL_mutex_lock(mutex);
  return RC_OK;
}

static void
UTL_cond_win32_notify(cond)
     UTL_COND * cond;
{
  /*PulseEvent(((HANDLE *)cond->data)[0]);*/
	PulseEvent((HANDLE)cond->data);
}

static void
UTL_cond_win32_notify_all(cond)
     UTL_COND * cond;
{
  /*PulseEvent(((HANDLE *)cond->data)[1]);*/
	PulseEvent((HANDLE)cond->data);
}

static void
UTL_cond_win32_destroy(cond)
     UTL_COND * cond;
{
	/*
  CloseHandle(((HANDLE *)cond->data)[0]);
  CloseHandle(((HANDLE *)cond->data)[1]);
  free(cond->data);
  */
	CloseHandle((HANDLE)cond->data);
}

static UTL_COND_METH win32_method = {
  UTL_cond_win32_init,
  UTL_cond_win32_wait,
  UTL_cond_win32_wait_timed,
  UTL_cond_win32_notify,
  UTL_cond_win32_notify_all,
  UTL_cond_win32_destroy
};

UTL_COND_METH *
UTL_cond_win32_method()
{
  return &win32_method;
}
#endif /* WIN32 */
