/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

#include <stdlib.h>
#include "UTL_mutex.h"

static UTL_MUTEX_METH * default_mutex_method = NULL;

void
UTL_mutex_set_default_method(meth)
     UTL_MUTEX_METH * meth;
{
  default_mutex_method = meth;
}

int
UTL_mutex_init_method(mutex, meth)
     UTL_MUTEX * mutex;
     UTL_MUTEX_METH * meth;
{
  if(meth == NULL) {
    if(default_mutex_method == NULL) {
      /* Specify in order of preference for default */
#ifdef PTHREADS
      default_mutex_method = UTL_mutex_pthread_method();
#elif defined(WIN32)
      default_mutex_method = UTL_mutex_win32_method();
#elif defined(STHREADS)
      default_mutex_method = UTL_mutex_sthread_method();
#else
      default_mutex_method = UTL_mutex_null_method();
#endif
    }
    mutex->method = default_mutex_method;
  }
  else
    mutex->method = meth;

  mutex->data = NULL;

  return mutex->method->init(mutex);
}

int
UTL_mutex_init(mutex)
     UTL_MUTEX * mutex;
{
  return UTL_mutex_init_method(mutex, NULL);
}

void
UTL_mutex_lock(mutex)
     UTL_MUTEX * mutex;
{
  mutex->method->lock(mutex);
}

void
UTL_mutex_unlock(mutex)
     UTL_MUTEX * mutex;
{
  mutex->method->unlock(mutex);
}

void
UTL_mutex_destroy(mutex)
     UTL_MUTEX * mutex;
{
  mutex->method->destroy(mutex);
}
