/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

#ifdef WIN32
#include <windows.h>
#include "RC.h"
#include "UTL_mutex.h"
#ifndef NOTRACE
#include <stdio.h>
#endif

/* win32 - based mutexes */

static int
UTL_mutex_win32_init(mutex)
     UTL_MUTEX * mutex;
{
  if((mutex->data = CreateMutex(NULL, FALSE, NULL)) == NULL)
    return RC_ERROR;
  /*
	if((mutex->data = malloc(sizeof(CRITICAL_SECTION))) == NULL)
		return RC_ERROR;
	InitializeCriticalSection((LPCRITICAL_SECTION) mutex->data);
	*/
  return RC_OK;
}

static void
UTL_mutex_win32_lock(mutex)
     UTL_MUTEX * mutex;
{
  WaitForSingleObject((HANDLE)mutex->data, INFINITE);
	/*printf("Locking %08x... ", mutex->data);fflush(stdout);*/
	/*EnterCriticalSection((LPCRITICAL_SECTION) mutex->data);*/
	/*printf("locked %08x\n", mutex->data);*/
}

static void
UTL_mutex_win32_unlock(mutex)
     UTL_MUTEX * mutex;
{
  ReleaseMutex((HANDLE)mutex->data);
	/*printf("Unlocking %08x... ", mutex->data);fflush(stdout);*/
	/*LeaveCriticalSection((LPCRITICAL_SECTION) mutex->data);*/
	/*printf("unlocked %08x\n", mutex->data);*/
}

static void
UTL_mutex_win32_destroy(mutex)
     UTL_MUTEX * mutex;
{
  CloseHandle((HANDLE)mutex->data);
  /*
	DeleteCriticalSection((LPCRITICAL_SECTION) mutex->data);
	free(mutex->data);
	*/
}

static UTL_MUTEX_METH win32_method = {
  UTL_mutex_win32_init,
  UTL_mutex_win32_lock,
  UTL_mutex_win32_unlock,
  UTL_mutex_win32_destroy
};

UTL_MUTEX_METH *
UTL_mutex_win32_method()
{
  return &win32_method;
}
#endif /* WIN32 */
