/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

#ifdef WIN32
#include <windows.h>
#include "RC.h"
#include "UTL_thread.h"

/* WIN32 threads implementation */

static int
UTL_thread_win32_create(thread, proc, arg)
     UTL_THREAD * thread;
     void (* proc)();
     void * arg;
{
	DWORD threadid;

	if((thread->data = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE) proc, arg, 0, &threadid)) == NULL)
		return RC_ERROR;
	return RC_OK;
}

static void
UTL_thread_win32_join(thread)
     UTL_THREAD * thread;
{
	DWORD termCode;

	for(;;) {
		if(!GetExitCodeThread((HANDLE) thread->data, &termCode))
			return;		/* Error */
		if(termCode == STILL_ACTIVE)
			WaitForSingleObject((HANDLE) thread->data, INFINITE);
		else
			return;		/* Success */
	}
}

static void
UTL_thread_win32_stop(thread)
     UTL_THREAD * thread;
{
	TerminateThread((HANDLE) thread->data, 0);
}

static void
UTL_thread_win32_destroy(thread)
     UTL_THREAD * thread;
{
  CloseHandle((HANDLE) thread->data);
}

static unsigned int
UTL_thread_win32_current_id()
{
  return GetCurrentThreadId();
}

static UTL_THREAD_METH win32_method = {
  UTL_thread_win32_create,
  UTL_thread_win32_join,
  UTL_thread_win32_stop,
  UTL_thread_win32_destroy,
  UTL_thread_win32_current_id
};

UTL_THREAD_METH *
UTL_thread_win32_method()
{
  return &win32_method;
}
#endif /* WIN32 */
