/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */



/*------------------------------------------------------------------------*/
/*--									--*/
/*--	ADM.h								--*/
/*--									--*/
/*--	Header file for administration program				--*/
/*------------------------------------------------------------------------*/

#ifndef ADM_H
#define ADM_H


/*--------------------------------*/
/*--  ADM Structures		  */
/*--------------------------------*/

/* ADM_CTX */

typedef struct adm_ctx {

  /* Streams */
  COM_STREAM **streams;
  int numServers;

  COM_CTX *comctx;
  UTL_CTX *utlctx;

  /* Command line arguments */
  int	opcode;				/* What to do.			*/
  int   server;                         /* Which server to apply opcode to */
  char	*keysize;			/* Size of key to generate 	*/
  int   threshold;                      /* sharing threshold            */
  char	*keyname;			/* Key name			*/
  char	*password;			/* Password			*/
  char	*ConfigFile;			/* Config file			*/
  int   version;                        /* The version of the key       */
  int   me;                             /* The admin ID number          */

} ADM_CTX;


/*------------------------*/
/*--  Naming Constants  --*/
/*------------------------*/

#define ADM_op_genkey		100		/* Generate a shared key	*/
#define ADM_op_shutdown		101		/* Shutdown servers		*/
#define ADM_op_refresh 		103		/* Refresh shares		*/
#define ADM_op_start		104		/* Start share servers:		*/
						/*	load keys, etc.		*/
#define ADM_op_suspend		105		/* Suspend share servers.	*/
#define ADM_op_corrupt		106		/* Send back incorrect responses*/
#define ADM_op_uncorrupt	107		/* Send back correct responses  */
#define ADM_op_get_version      108
#define ADM_op_version_set      109
#define ADM_op_recover          110
#define ADM_op_get_pub_key      111
#define ADM_op_revert           112

#define ADM_HIGH                  8
#define ADM_LOW                  16


/*--------------------------------*/
/*--  Main function prototypes  --*/
/*--------------------------------*/

int ADM_ParseCmdLine(ADM_CTX *ctx, int argc, char **argv);

int ADM_control(ADM_CTX *ctx);
int ADM_GenKey(ADM_CTX *ctx);
int ADM_delete(ADM_CTX *ctx);
int ADM_refresh(ADM_CTX *ctx);
int ADM_recover(ADM_CTX *ctx, int type);
int ADM_GetPublicKey(ADM_CTX *ctx, BIGNUM *inN, BIGNUM *inPubExp);
int ADM_SendHeader(ADM_CTX *ctx);
int ADM_RefPubKey(ADM_CTX *ctx);
int ADM_CheckEnd(ADM_CTX *ctx);
#endif


