/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

/************************************************************** */
/*								*/
/*	IT_bn.h							*/
/*								*/
/*	Macro's for calling the BN functions with error		*/
/*	checking.						*/
/*								*/
/*	Also, header file for extended BN functions.		*/
/*								*/
/************************************************************** */

#ifndef IT_BN
#define IT_BN

/*--------  IT Headers  --------------*/

int IT_mod_exp_NEG(BIGNUM *r, BIGNUM *a, BIGNUM *p, BIGNUM *m, BN_CTX *ctx);
//int IT_add(BIGNUM *a, BIGNUM *b, BIGNUM *c);
//int IT_sub(BIGNUM *a, BIGNUM *b, BIGNUM *c);


/*--------  BN Macros  ---------------*/

#define IT_copy(a,b)    if (!BN_copy((a), (b))) {			\
    UTL_Log(__FILE__, LOG_FATAL, "BigNum Error. Line %d", __LINE__);	\
    return RC_ERROR;							\
  }

#define IT_zero(a)							\
  if (!BN_zero((a))) {							\
    UTL_Log(__FILE__, LOG_FATAL, "BigNum Error. Line %d", __LINE__);	\
    return RC_ERROR;							\
  }

#define IT_add(a,b,c)							\
  if (!BN_add((a), (b), (c))) {			\
    UTL_Log(__FILE__, LOG_FATAL, "BigNum Error. Line %d", __LINE__);	\
    return RC_ERROR;							\
  }

#define IT_sub(a,b,c)							\
  if (!BN_sub((a), (b), (c))) {						\
    UTL_Log(__FILE__, LOG_FATAL, "BigNum Error. Line %d", __LINE__);	\
    return RC_ERROR;							\
  }

#define IT_mul(a,b,c,d)							\
  if (!BN_mul((a), (b), (c), (d))) {					\
    UTL_Log(__FILE__, LOG_FATAL, "BigNum Error. Line %d", __LINE__);	\
    return RC_ERROR;							\
  }

#define IT_div(a,b,c,d,e)						\
  if (!BN_div((a), (b), (c), (d), (e))) {				\
    UTL_Log(__FILE__, LOG_FATAL, "BigNum Error. Line %d", __LINE__);	\
    return RC_ERROR;							\
  }

#define IT_mod(a,b,c,d)							\
  if (!BN_mod((a), (b), (c), (d))) {					\
    UTL_Log(__FILE__, LOG_FATAL, "BigNum Error. Line %d", __LINE__);	\
    return RC_ERROR;							\
  }

#define IT_mod_mul(a,b,c,d,e)						\
  if (!BN_mod_mul((a), (b), (c), (d), (e))) {				\
    UTL_Log(__FILE__, LOG_FATAL, "BigNum Error. Line %d", __LINE__);	\
    return RC_ERROR;							\
  }

#define IT_mod_exp(a,b,c,d,e)						\
  if (!BN_mod_exp((a), (b), (c), (d), (e))) {				\
    UTL_Log(__FILE__, LOG_FATAL, "BigNum Error. Line %d", __LINE__);	\
    return RC_ERROR;							\
  }

#define IT_mod_exp_neg(a,b,c,d,e)					\
  if (!IT_mod_exp_NEG((a), (b), (c), (d), (e))) {			\
    UTL_Log(__FILE__, LOG_FATAL, "BigNum Error. Line %d", __LINE__);	\
    return RC_ERROR;							\
  }

#define IT_set_word(a,b)						\
  if (!BN_set_word((a), (b))) {						\
    UTL_Log(__FILE__, LOG_FATAL, "BigNum Error. Line %d", __LINE__);	\
    return RC_ERROR;							\
  }

#define IT_add_word(a,b)						\
  if (!BN_add_word((a), (b))) {						\
    UTL_Log(__FILE__, LOG_FATAL, "BigNum Error. Line %d", __LINE__);	\
    return RC_ERROR;							\
  }


#endif
