/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

/* Platform-independent interface to condition variables */

#ifndef _UTL_COND_H_
#define _UTL_COND_H_

#include "UTL_mutex.h"

typedef struct utl_cond_st {
  struct utl_cond_meth_st {
    int (* init)(struct utl_cond_st *);
    void (* wait)(struct utl_cond_st *, UTL_MUTEX *);
    int (* wait_timed)(struct utl_cond_st *, UTL_MUTEX *, int);
    void (* notify)(struct utl_cond_st *);
    void (* notify_all)(struct utl_cond_st *);
    void (* destroy)(struct utl_cond_st *);
  } * method;
  void * data;
} UTL_COND;

typedef struct utl_cond_meth_st UTL_COND_METH;

extern void UTL_cond_set_default_method(UTL_COND_METH *);
extern int UTL_cond_init_method(UTL_COND *, UTL_COND_METH *);

extern UTL_COND_METH * UTL_cond_null_method();
#ifdef PTHREADS
extern UTL_COND_METH * UTL_cond_pthread_method();
#endif
#ifdef STHREADS
extern UTL_COND_METH * UTL_cond_sthread_method();
#endif
#ifdef WIN32
extern UTL_COND_METH * UTL_cond_win32_method();
#endif

/* Primary API */
extern int UTL_cond_init(UTL_COND *);
extern void UTL_cond_wait(UTL_COND *, UTL_MUTEX *);
extern int UTL_cond_wait_timed(UTL_COND *, UTL_MUTEX *, int);
extern void UTL_cond_notify(UTL_COND *);
extern void UTL_cond_notify_all(UTL_COND *);
extern void UTL_cond_destroy(UTL_COND *);

#endif
