/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

/* Platform-independent thread abstraction */

#ifndef _UTL_THREAD_H_
#define _UTL_THREAD_H_

typedef struct utl_thread_st {
  struct utl_thread_meth_st {
    int (* create)(struct utl_thread_st *, void (*)(void *), void *);
    void (* join)(struct utl_thread_st *);
    void (* stop)(struct utl_thread_st *);
    void (* destroy)(struct utl_thread_st *);
    unsigned int (* current_id)();
  } * method;
  void * data;
} UTL_THREAD;

typedef struct utl_thread_meth_st UTL_THREAD_METH;

extern void UTL_thread_set_default_method(UTL_THREAD_METH *);
extern int UTL_thread_create_method(UTL_THREAD *, UTL_THREAD_METH *, void (*)(void *), void *);
extern unsigned int UTL_thread_current_id_method(UTL_THREAD_METH *);

extern UTL_THREAD_METH * UTL_thread_null_method();
#ifdef PTHREADS
extern UTL_THREAD_METH * UTL_thread_pthread_method();
#endif
#ifdef STHREADS
extern UTL_THREAD_METH * UTL_thread_sthread_method();
#endif
#ifdef WIN32
extern UTL_THREAD_METH * UTL_thread_win32_method();
#endif

/* Primary API */
extern int UTL_thread_create(UTL_THREAD *, void (*)(void *), void *);
extern void UTL_thread_join(UTL_THREAD *);
extern void UTL_thread_stop(UTL_THREAD *);
extern void UTL_thread_destroy(UTL_THREAD *);
extern unsigned int UTL_thread_current_id();

#endif
