import java.applet.Applet;
import java.awt.*;

public class MonitorApplet extends Applet
  implements StateChangeHandler, DestroyHandler {
  protected MonitorPanel mp;
  protected Button b_conn, b_disconn, b_atdetach;
  protected Label l_filler;
  protected MonitorFrame f_detach;

  public void init() {
    setLayout(new BorderLayout());

    Panel p = new Panel();
    p.setLayout(new GridLayout(1, 0));
    b_conn = new Button("Connect");
    p.add(b_conn);
    b_disconn = new Button("Disconnect");
    p.add(b_disconn);
    b_atdetach = new Button("Detach");
    p.add(b_atdetach);

    add("North", p);

    l_filler = new Label("ITTC Activity Monitor running in separate frame",
			 Label.CENTER);

    mp = new MonitorPanel();
    mp.setHostEditable(false);

    String startup = getParameter("startup");
    if(startup != null && startup.equalsIgnoreCase("frame")) {
      b_atdetach.setLabel("Re-Attach");
      f_detach = new MonitorFrame(mp, this);
      f_detach.show();
      add("Center", l_filler);
    }
    else {
      add("Center", mp);
      mp.setHandler(this);
    }

    stateChangeNotify();

    try {
      int port = 0;
      String port_s = getParameter("port");
      if(port_s != null) {
	try {
	  port = Integer.parseInt(port_s);
	}
	catch(NumberFormatException e) {}
      }
      mp.connect(getDocumentBase().getHost(), port);
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public void start() {
    if(f_detach == null) {
      try {
	mp.connect();
      } catch(Exception e) {
	e.printStackTrace();
      }
    }
  }

  public void stop() {
    if(f_detach == null)
      mp.disconnect();
  }

  public void stateChangeNotify() {
    if(f_detach != null) {
      b_conn.disable();
      b_disconn.disable();
    }
    else if(mp.connected()) {
      b_conn.disable();
      b_disconn.enable();
    }
    else {
      b_conn.enable();
      b_disconn.disable();
    }
  }

  public void destroyNotify() {
    if(f_detach != null) {
      f_detach = null;
      remove(l_filler);
      add("Center", mp);
      validate();
      mp.setHandler(this);
      b_atdetach.setLabel("Detach");
      stateChangeNotify();
      if(!isActive())
	mp.disconnect();
    }
  }

  public void detach() {
    remove(mp);
    add("Center", l_filler);
    validate();
    b_atdetach.setLabel("Re-Attach");
    f_detach = new MonitorFrame(mp, this);
    f_detach.show();
    stateChangeNotify();
  }

  public boolean action(Event evt, Object arg) {
    if(evt.target != null) {
      try {
	if(evt.target == b_conn)
	  mp.connect();
	else if(evt.target == b_disconn)
	  mp.disconnect();
	else if(evt.target == b_atdetach) {
	  if(f_detach == null) {
	    detach();
	  }
	  else
	    f_detach.close();
	}
      }
      catch(Exception e) {
	e.printStackTrace();
      }
    }
    return super.action(evt, arg);
  }
}
