import java.awt.*;

public class MonitorFrame extends Frame implements StateChangeHandler {
  protected MonitorPanel mp;
  protected Menu m_file;
  protected MenuItem m_exit, m_conn, m_disconn;
  protected DestroyHandler killHandler;
  protected boolean disconnectOnClose;

  private MonitorFrame(MonitorPanel child) {
    super("ITTC Activity Monitor");

    mp = child;
    mp.setHandler(this);
    add("Center", mp);
    mp.validate();

    MenuBar mb = new MenuBar();

    m_file = new Menu("File");
    m_conn = new MenuItem("Connect");
    m_file.add(m_conn);
    m_disconn = new MenuItem("Disconnect");
    m_file.add(m_disconn);
    mb.add(m_file);

    setMenuBar(mb);
    resize(500, 300);
  }

  public MonitorFrame(MonitorPanel child, DestroyHandler h) {
    this(child);

    m_file.addSeparator();
    m_exit = new MenuItem("Delete");
    m_file.add(m_exit);

    disconnectOnClose = false;
    setDestroyHandler(h);
    stateChangeNotify();
  }

  public MonitorFrame(MonitorPanel child, String host, int port) {
    this(child);

    m_file.addSeparator();
    m_exit = new MenuItem("Exit");
    m_file.add(m_exit);

    disconnectOnClose = true;
    stateChangeNotify();
    try {
      mp.connect(host, port);
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public void setDestroyHandler(DestroyHandler h) {
    killHandler = h;
  }

  public void stateChangeNotify() {
    if(mp.connected()) {
      m_conn.disable();
      m_disconn.enable();
    }
    else {
      m_conn.enable();
      m_disconn.disable();
    }
  }

  public void close() {
    if(disconnectOnClose)
      mp.disconnect();
    if(killHandler != null) {
      remove(mp);
      killHandler.destroyNotify();
    }
    hide();
    dispose();
  }

  public boolean action(Event evt, Object arg) {
    if(evt.target != null) {
      try {
	if(evt.target == m_exit)
	  close();
	else if(evt.target == m_conn)
	  mp.connect();
	else if(evt.target == m_disconn)
	  mp.disconnect();
      }
      catch(Exception e) {
	e.printStackTrace();
      }
    }
    return super.action(evt, arg);
  }

  public boolean handleEvent(Event evt) {
    if(evt.id == Event.WINDOW_DESTROY) {
      close();
    }
    return super.handleEvent(evt);
  }

  public static void main(String[] args) {
    MonitorFrame mf = new MonitorFrame(new MonitorPanel(),
				       "ittc.stanford.edu", 6998);
    mf.setDestroyHandler(new ExitOnClose());
    mf.show();
  }
}

class ExitOnClose implements DestroyHandler {
  public void destroyNotify() {
    System.exit(0);
  }
}
