import java.awt.*;
import java.io.*;
import java.net.*;

public class MonitorPanel extends Panel implements Runnable {
  protected TextArea text;
  protected TextField t_host;
  protected TextField t_port;
  protected Socket insock;
  protected Thread daemon;
  protected StateChangeHandler connHandler;

  public MonitorPanel() {
    setLayout(new BorderLayout());
    text = new TextArea();
    add("Center", text);

    Panel p = new Panel();

    p.add(new Label("Host"));
    t_host = new TextField(30);
    p.add(t_host);
    p.add(new Label("Port"));
    t_port = new TextField(5);
    p.add(t_port);

    add("South", p);

    insock = null;
    daemon = null;
  }

  public void setHandler(StateChangeHandler newHandler) {
    connHandler = newHandler;
  }

  public void setHostEditable(boolean b) {
    t_host.setEditable(b);
  }

  public boolean connected() {
    if(insock == null || daemon == null)
      return false;
    else
      return true;
  }

  public void connect(String host, int port) throws IOException {
    t_host.setText(host);
    t_port.setText(String.valueOf(port));
    if(port <= 0)
      return;

    if(insock != null)
      disconnect();
    insock = new Socket(host, port);
    daemon = new Thread(this);
    daemon.start();

    text.appendText("Connected to " + host + ":" + String.valueOf(port) +
		    "...\n");
  }

  public void connect() throws IOException {
    if(insock != null)
      return;
    try {
      String host = t_host.getText();
      if(host == null || host.length() == 0)
	return;
      int port = Integer.parseInt(t_port.getText());
      insock = new Socket(host, port);
      daemon = new Thread(this);
      daemon.start();
    }
    catch(NumberFormatException e) {}
  }

  public void disconnect() {
    if(daemon != null) {
      daemon.stop();
      daemon = null;
    }
    if(insock != null) {
      try { insock.close(); } catch(IOException e) {}
      insock = null;
    }

    if(connHandler != null)
      connHandler.stateChangeNotify();
  }

  public void run() {
    try {
      InputStream in = insock.getInputStream();
      DataInputStream din = new DataInputStream(in);
      String line;

      if(connHandler != null)
	connHandler.stateChangeNotify();

      while((line = din.readLine()) != null)
	text.appendText(line + "\n");
    }
    catch(IOException e) {}

    disconnect();
  }

  public void finalize() throws Throwable {
    disconnect();
    super.finalize();
  }
}
