#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#define HAVE_RLIMIT 1

#ifdef HAVE_RLIMIT
#include <sys/resource.h>
#endif

#include "ITTC.h"

#define DEFAULT_PORT	7000
#define DATA_PORT	6999
#define TEXT_PORT	6998

#define DATA_FD		1
#define TEXT_FD		2

struct monitor_pkt {
  int peerId;

  char time[HMSMDY_SIZE + 1];
  int msgattr;
  int msglen;

  char * msg;
};

main(argc, argv)
     int argc;
     char * argv[];
{
  struct sockaddr_in saddr;
  fd_set readfds;
  in_port_t port;
  int insock, dsock, tsock;
  int len, alen;
  unsigned char buf[256];
  unsigned char * p;
  struct monitor_pkt pkt;
  char peerString[64];
  char textOut[256];
  char * ofdvec;
  int maxfd, limfd;
  int i;

#ifdef HAVE_RLIMIT
  struct rlimit rl;

  getrlimit(RLIMIT_NOFILE, &rl);
  limfd = rl.rlim_cur;
#elif defined(NOFILE)
  limfd = NOFILE;
#else
  limfd = 20;
#endif
  printf("Using pool of %d descriptors\n", limfd);
  ofdvec = malloc(sizeof(char) * limfd);
  memset(ofdvec, 0, sizeof(char) * limfd);

  if(argc > 1)
    port = atoi(argv[1]);
  else
    port = DEFAULT_PORT;

  if((insock = socket(AF_INET, SOCK_DGRAM, 0)) < 0 ||
     (dsock = socket(AF_INET, SOCK_STREAM, 0)) < 0 ||
     (tsock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    perror("socket");
    exit(1);
  }
  maxfd = insock;
  if(dsock > maxfd)
    maxfd = dsock;
  if(tsock > maxfd)
    maxfd = tsock;

  saddr.sin_family = AF_INET;
  saddr.sin_addr.s_addr = INADDR_ANY;

  saddr.sin_port = htons(port);
  if(bind(insock, (struct sockaddr *) &saddr, sizeof(saddr)) < 0) {
    perror("bind (incoming)");
    exit(1);
  }

  saddr.sin_port = htons(DATA_PORT);
  if(bind(dsock, (struct sockaddr *) &saddr, sizeof(saddr)) < 0) {
    perror("bind (data)");
    exit(1);
  }

  saddr.sin_port = htons(TEXT_PORT);
  if(bind(tsock, (struct sockaddr *) &saddr, sizeof(saddr)) < 0) {
    perror("bind (text)");
    exit(1);
  }

  if(listen(dsock, 1) < 0 || listen(tsock, 1) < 0) {
    perror("listen");
    exit(2);
  }

  for(;;) {
    FD_ZERO(&readfds);
    FD_SET(insock, &readfds);
    FD_SET(dsock, &readfds);
    FD_SET(tsock, &readfds);

    for(i = 0; i <= maxfd; ++i)
      if(ofdvec[i])
	FD_SET(i, &readfds);
    
    if(select(maxfd + 1, &readfds, NULL, NULL, NULL) < 0)
      break;

    for(i = 0; i <= maxfd; ++i)
      if(ofdvec[i] && FD_ISSET(i, &readfds)) {
	len = read(i, buf, sizeof(buf));
	printf("closing fd %d\n", i);
	close(i);
	ofdvec[i] = 0;
      }

    if(FD_ISSET(dsock, &readfds)) {
      alen = sizeof(saddr);
      i = accept(dsock, (struct sockaddr *) &saddr, &alen);
      if(i < 0)
	perror("accept (data)");
      else {
	/*shutdown(i, 0);*/
	printf("Accepted fd %d (data)\n", i);
	if(i > maxfd)
	  maxfd = i;
	ofdvec[i] = DATA_FD;
      }
    }
    if(FD_ISSET(tsock, &readfds)) {
      alen = sizeof(saddr);
      i = accept(tsock, (struct sockaddr *) &saddr, &alen);
      if(i < 0)
	perror("accept (text)");
      else {
	/*shutdown(i, 0);*/
	printf("Accepted fd %d (text)\n", i);
	if(i > maxfd)
	  maxfd = i;
	ofdvec[i] = TEXT_FD;
      }
    }
    if(FD_ISSET(insock, &readfds)) {
      len = recvfrom(insock, buf, sizeof(buf), 0, (struct sockaddr *) &saddr, &alen);
      if(len < 0)
	break;
      if(len < 16) {
	fprintf(stderr, "Packet too short (len = %d)\n", len);
	continue;
      }

      p = buf;

      pkt.peerId = ntohl(*(in_port_t *) p);
      p += sizeof(in_addr_t);
      len -= sizeof(in_addr_t);

      memcpy(pkt.time, p, HMSMDY_SIZE);
      pkt.time[HMSMDY_SIZE] = '\0';
      p += HMSMDY_SIZE;
      len -= HMSMDY_SIZE;

      pkt.msgattr = ntohl(*(in_addr_t *) p);
      p += sizeof(in_addr_t);
      len -= sizeof(in_addr_t);

      pkt.msglen = ntohl(*(in_addr_t *) p);
      p += sizeof(in_addr_t);
      len -= sizeof(in_addr_t);

      if(len < pkt.msglen) {
	fprintf(stderr, "Packet too short (%d < %d)\n", len, pkt.msglen);
	continue;
      }

      pkt.msg = p;

      COM_unparse_peer_info(peerString, pkt.peerId);

      sprintf(textOut, "%s:%s(%d):%.*s\n", peerString, pkt.time,
	      pkt.msgattr, pkt.msglen, pkt.msg);
      for(i = 0; i <= maxfd; ++i)
	if(ofdvec[i] == TEXT_FD) {
	  if(send(i, textOut, strlen(textOut), 0) < 0)
	    perror("send");
	}
    }
  }
}
