#!/bin/sh
# Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
# All rights reserved.
#
# This package is an implementation of the ITTC system written
# by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
# 
# This library is free for non-commercial use as long as
# the following conditions are adhered to.  The following conditions
# apply to all source code found in the src directory of this distribution.
# Any source code found in other directories is subject to the copyright 
# notice within that code.
# 
# Copyright belongs to Stanford University and as such any Copyright 
# notices in the code are not to be removed.  This package may not be 
# used in a commercial product without the explicit consent of the authors. 
#
# Free redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgment:
#    "This product includes software written by Mike Malkin, Tom Wu, 
#     and Dan Boneh."
# 
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
# 
# The license and distribution terms for any publically available version or
# derivative of this code cannot be changed.  i.e. this code cannot simply be
# copied and put under another distribution license
# [including the GNU Public License.]
#

BINDIR=`dirname $0`
PDIR=$BINDIR/..
USAGE="Usage: $0 [ -initCA | -makecert ]
       -initCA    Re-initialize the ITTC internal CA
       -makecert  Create a cert/key pair for an ITTC peer (client,server,admin)"

if [ $# != 1 ]
then
    echo "$USAGE"
    exit 1
fi

case $1 in
-initCA)
    echo "Are you sure you want to re-initialize the local CA (y/n)?"
    read yn
    case $yn in
    y*) ;;
    *) echo "Aborted."; exit 1
    esac
    /bin/rm -rf "$PDIR"/localCA/*.bak
    mv "$PDIR/localCA/certs" "$PDIR/localCA/certs.bak"
    mv "$PDIR/localCA/crl" "$PDIR/localCA/crl.bak"
    mv "$PDIR/localCA/newcerts" "$PDIR/localCA/newcerts.bak"
    mv "$PDIR/localCA/private" "$PDIR/localCA/private.bak"
    mv "$PDIR/localCA/serial" "$PDIR/localCA/serial.bak"
    mv "$PDIR/localCA/index.txt" "$PDIR/localCA/index.bak"
    mv "$PDIR/localCA/cacert.pem" "$PDIR/localCA/cacert.bak"
    mkdir "$PDIR/localCA/certs" "$PDIR/localCA/newcerts" "$PDIR/localCA/crl" "$PDIR/localCA/private"
    echo "01" > "$PDIR/localCA/serial"
    cp /dev/null "$PDIR/localCA/index.txt"
    echo "***"
    echo "*** Creating new local CA - please enter a passphrase when"
    echo "*** prompted.  This passphrase will protect the local CA"
    echo "***"
    $BINDIR/ittc_req -config "$PDIR/localCA/local.cnf" -newkey rsa:1024 -keyout "$PDIR/localCA/private/cakey.pem" -x509 -days 365 -out "$PDIR/localCA/cacert.pem"
    ;;
-makecert)
    echo "Enter type of cert to create [(c)lient,(s)server,(a)dmin]:"
    read CTYPE
    case $CTYPE in
    c*) prefix="client"; CN="CLIENT";;
    s*) prefix="server"; CN="SERVER";;
    a*) prefix="admin"; CN="ADMIN";;
    *)  echo "Unknown cert type requested"; exit 1
    esac
    echo "Enter the number of the entity (0-5):"
    read CNUM
    case $CNUM in
    [0-6]) ;;
    *) echo "Invalid entity number"; exit 1
    esac
    CDIR=$PDIR/$prefix$CNUM/certs
    echo "***"
    echo "*** Please enter [$CN $CNUM] for the Common Name in this cert"
    echo "***"
    $BINDIR/ittc_req -config "$PDIR/localCA/local.cnf" -newkey rsa:768 -nodes -keyout "$CDIR/com-key.pem" -out "$CDIR/com-req.pem"
    echo "***"
    echo "*** Certifying request - please enter your local CA's"
    echo "*** passphrase when prompted"
    echo "***"
    $BINDIR/ittc_ca -config "$PDIR/localCA/local.cnf" -in "$CDIR/com-req.pem" -out "$CDIR/com-cert.pem"
    ;;
*)
    echo "$USAGE"
    exit 1
esac
