/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/time.h>
#include "ITTC.h"

#define STRING_LENGTH     20


int main(argc, argv)
     int argc;
     char * argv[];
{
  UTL_STATS *stats;
  RSA_SHARED_KEY * key;
  float tempf;
  unsigned char * buf;
  unsigned char * buf2;
  int buflen, r;
  char txt[256];
  int i, j;
  int good = 0;
  int iterations;


  /* Initialize */
  if(argc < 3) {
    fprintf(stderr, "Usage:  %s <shared-key> <iterations> <threads>\n", *argv);
    exit(1);
  }
  iterations = atoi(argv[2]);
  srand(99);

  UTL_InitStats(0, 1, stats);
  UTL_NameTimeStat(0, "Total Time", stats);
  UTL_StartStat(0, stats);

  for (i=0; i<iterations; i++) {

    fprintf(stderr, "\n[%4d ]------------------------------------------------------------\n", i);

    /* "key" now behaves just like a "normal" RSA public/private key. */
    key = ITTC_load_RSA_key(argv[1]);
    if(key == NULL) {
      fprintf(stderr, "%s: Unable to read RSA key '%s'\n", *argv, argv[1]);
      exit(2);
    }
    
    buflen = RSA_size(key);
    if((buf = malloc(buflen)) == NULL || (buf2 = malloc(buflen)) == NULL) {
      fprintf(stderr, "%s: Unable to allocate memory\n", *argv);
      exit(3);
    }

    /* Make a random text string */
    for (j=0; j<STRING_LENGTH; j++) txt[j] = (rand() % 26) + 'a';
    txt[STRING_LENGTH] = '\0';
    printf("%-20s: %s\n", "Original message", txt);
    
    /* Uses EAY padding/encryption (local) */
    if((r = RSA_public_encrypt(strlen(txt) + 1, txt, buf, key, RSA_PKCS1_PADDING)) < 0) {
      fprintf(stderr, "%s: Error encrypting data\n", *argv);
      exit(5);
    }
    
    /* Performs shared decryption over network */
    if((r = RSA_private_decrypt(r, buf, buf2, key, RSA_PKCS1_PADDING)) < 0) {
      fprintf(stderr, "%s: Error decrypting data\n", *argv);
      exit(6);
    }
    printf("%-20s: %s\n", "RSA Decrypted", buf2);

    /* Check the result */
    if (strcmp(buf2, txt) == 0) {
      good++;
    }

    free(buf);
    free(buf2);
    KEY_RSA_shared_key_free(key);    
  }

  UTL_FinishStat(0, stats);
  printf("\n%d good iterations, %d bad iterations\n", good, iterations - good);

  tempf = (float) UTL_TimeStat(0, stats);
  printf("Total time for %d decryptions: %0.1fms\n", iterations, tempf);

  tempf = (float) UTL_TimeStat(0, stats) / iterations / 1000;
  printf("Average time per decryption: %0.1fms\n", tempf);

  tempf = (float) 1000000 * iterations / UTL_TimeStat(0, stats);
  printf("Decryptions per second: %0.3f\n\n", tempf);
  
  return 0;
}

