/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

#include "KEY.h"

int
main(argc, argv)
     int argc;
     char * argv[];
{
  RSA_SHARED_KEY * key;
  int i;

  if(argc < 2) {
    fprintf(stderr, "Usage: %s filename\n", argv[0]);
    exit(1);
  }

  ITTC_init();
  key = KEY_RSA_shared_key_read(argv[1]);

  if(key == NULL) {
    fprintf(stderr, "Unable to read key from %s\n", argv[1]);
    exit(3);
  }

  printf("n = ");
  BN_print_fp(stdout, KEY_RSA_shared_key_n(key));
  printf("\ne = ");
  BN_print_fp(stdout, KEY_RSA_shared_key_e(key));
  printf("\nParameters: threshold=%d out of %d",
	 KEY_RSA_shared_key_threshold(key), KEY_RSA_shared_key_nparties(key));
  printf("\ng = ");
  BN_print_fp(stdout, KEY_RSA_shared_key_g(key));
  printf("\nNumber of g^d values: %d\n", KEY_RSA_shared_key_k(key));
  for(i = 0; i < KEY_RSA_shared_key_k(key); ++i) {
    printf("g^d[%d] = ", i);
    BN_print_fp(stdout, KEY_RSA_shared_key_gd(key, i));
    printf("\n");
  }
  KEY_RSA_shared_key_free(key);

  return 0;
}
