/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

#include <stdio.h>
#include "ITTC.h"

void
hexprint(s, len)
     unsigned char * s;
     int len;
{
  while(len-- > 0)
    printf("%02X", *s++);
}

int
main(argc, argv)
     int argc;
     char * argv[];
{
  RSA_SHARED_KEY * key;
  unsigned char * buf;
  unsigned char * buf2;
  int buflen, r;

  if(argc < 3) {
    fprintf(stderr, "Usage:  %s <shared-key> <message-string>\n", *argv);
    exit(1);
  }

  key = ITTC_load_RSA_key(argv[1]);
  if(key == NULL) {
    fprintf(stderr, "%s: Unable to read RSA key '%s'\n", *argv, argv[1]);
    exit(2);
  }

  /*** "key" now behaves just like a "normal" RSA public/private key. ***/

  buflen = RSA_size(key);
  if((buf = malloc(buflen)) == NULL || (buf2 = malloc(buflen)) == NULL) {
    fprintf(stderr, "%s: Unable to allocate memory\n", *argv);
    exit(3);
  }

  printf("Original message (hex): ");
  hexprint(argv[2], strlen(argv[2]) + 1);
  printf("\n");

  /*** Uses EAY padding/encryption (local) ***/
  if((r = RSA_public_encrypt(strlen(argv[2]) + 1, argv[2], buf, key, RSA_PKCS1_PADDING)) < 0) {
    fprintf(stderr, "%s: Error encrypting data\n", *argv);
    ERR_print_errors_fp(stderr);
    exit(5);
  }

  printf("RSA Encrypted: ");
  hexprint(buf, r);
  printf("\n");

  /*** Performs shared decryption over network ***/
  if((r = RSA_private_decrypt(r, buf, buf2, key, RSA_PKCS1_PADDING)) < 0) {
    fprintf(stderr, "%s: Error decrypting data\n", *argv);
    ERR_print_errors_fp(stderr);
    exit(6);
  }

  printf("RSA Decrypted: ");
  hexprint(buf2, r);
  printf(" (\"%s\")\n", buf2);

  free(buf);
  free(buf2);

  return 0;
}
