/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

#include <stdio.h>
#include "COM.h"
#include "UTL.h"

COM_STREAM * conns[32];
int nconns;
  BIGNUM * n;

void
read_ctx(arg)
	void * arg;
{
	int i;
	COM_CTX * ctx = (COM_CTX *) arg;
  COM_INFO inf;
  void * p;

  while(nconns > 0) {
    i = COM_select(ctx, &p, COM_SELECT_READ | COM_SELECT_CLOSE);
    if(i == COM_SELECTED_READ) {
      if(COM_read_bn((COM_STREAM *) p, n) < 0)
		printf("Read error\n");
      else {
		COM_get_peer_info((COM_STREAM *) p, &inf);
		printf("Result from server %d: ", COM_INFO_GET_NUM(inf));
		BN_print_fp(stdout, n);
		printf("\n");
      }
    }
    else if(i == COM_SELECTED_CLOSE) {
      for(i = 0; i < nconns; ++i)
	if(conns[i] == (COM_STREAM *) p) {
#ifdef STATS
	  COM_print_stats(conns[i]);
#endif
	  /*COM_close(conns[i]);*/
	  conns[i] = conns[--nconns];
	  break;
	}
    }
    else if(i == COM_SELECTED_DONE) {
      printf("All streams closed, exiting...\n");
      break;
    }
    else {
      printf("Select returned unexpected value %d\n", i);
    }
  }
}

int
main(argc, argv)
     int argc;
     char * argv[];
{
  UTL_CTX * utl_ctx;
  UTL_THREAD thread;
  COM_CTX * ctx;
  int i, num, vport;
  char s[256];

  if(argc < 5) {
    fprintf(stderr, "Usage: %s config-file my-num port-num server-num ...\n", argv[0]);
    exit(1);
  }

  ITTC_init();
  printf("Using config-file %s\n", argv[1]);
  utl_ctx = UTL_init_ctx();
  UTL_LoadConfigFile(utl_ctx, argv[1]);
  UTL_OpenLogFile(utl_ctx, COM_TYPE_CLIENT, atoi(argv[2]));

  if((ctx = COM_init(utl_ctx, COM_TYPE_CLIENT, atoi(argv[2]))) == NULL) {
    fprintf(stderr, "COM_init() failed\n");
    exit(1);
  }
  vport = atoi(argv[3]);

  /*COM_ctx_set_flags(ctx, COM_FLAG_BLOCKING_CONNECT);*/

  n = BN_new();
  nconns = 0;
  for(i = 4; i < argc; ++i) {
    num = atoi(argv[i]);
    printf("Connecting to share-server #%d...\n", num);
    if((conns[nconns] = COM_connect(ctx, COM_TYPE_SHARE_SERVER, num, vport)) != NULL) {
      BN_set_word(n, i);
      printf("Connection state (before) = %d\n", conns[nconns]->state);
      /*
      COM_select_v(ctx, &p, &conns[nconns], 1, COM_SELECT_WRITE | COM_SELECT_CLOSE);
      printf("Connection state (after) = %d\n", conns[nconns]->state);
      */
      COM_write_bn(conns[nconns], n);
      ++nconns;
    }
  }

  printf("Successfully established %d connections\n", nconns);

  UTL_thread_create(&thread, read_ctx, ctx);

  while(1) {
    printf("Enter a number: ");
    fflush(stdout);
    if(gets(s) == NULL || strlen(s) == 0)
	  break;
    BN_ascii2bn(&n, s);
      for(i = 0; i < nconns; ++i) {
		if(conns[i] == NULL)
			continue;
		if(COM_write_bn(conns[i], n) < 0) {
			fprintf(stderr, "COM_write_bignum() failed\n");
		  COM_close(conns[i]);
			conns[i] = conns[--nconns];
		  continue;
		}
		printf("Stream %d: %d bytes pending\n", i, COM_write_pending(conns[i]));
      }
  }

  UTL_thread_stop(&thread);

#ifdef STATS
  for(i = 0; i < nconns; ++i)
    if(conns[i])
      COM_print_stats(conns[i]);
#endif

  COM_ctx_close(ctx);
  UTL_free_ctx(utl_ctx);
}
