/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

#include <stdio.h>
#include "COM.h"

int
main(argc, argv)
     int argc;
     char * argv[];
{
  UTL_CTX * utl_ctx;
  COM_CTX * ctx;
  COM_STREAM * str;
  COM_INFO inf;
  int i, num, vport, iterations;
  void * p;
  char s[256];
  BIGNUM * n;

  if(argc < 6) {
    fprintf(stderr, "Usage: %s config-file my-num port-num server-num iterations\n", argv[0]);
    exit(1);
  }

  printf("Using config-file %s\n", argv[1]);
  utl_ctx = UTL_init_ctx();
  UTL_LoadConfigFile(utl_ctx, argv[1]);
  UTL_OpenLogFile(utl_ctx, COM_TYPE_CLIENT, atoi(argv[2]));

  if((ctx = COM_init(utl_ctx, COM_TYPE_CLIENT, atoi(argv[2]))) == NULL) {
    fprintf(stderr, "COM_init() failed\n");
    exit(1);
  }
  vport = atoi(argv[3]);
  iterations = atoi(argv[5]);

  /*COM_ctx_set_flags(ctx, COM_FLAG_BLOCKING_CONNECT);*/

  n = BN_new();
  num = atoi(argv[4]);
  printf("Connecting to share-server #%d...\n", num);
  str = COM_connect(ctx, COM_TYPE_SHARE_SERVER, num, vport);
  if(str != NULL) {
    for(i = 0; i < iterations; ++i) {
      BN_rand(n, 64, 0, 0);
      if(BN_mod_word(n, 10) == 0) {
	COM_close(str);
#ifdef STATS
	COM_print_stats(str);
#endif
	str = COM_connect(ctx, COM_TYPE_SHARE_SERVER, num, vport);
      }
      if(COM_write_bn(str, n) < 0) {
	fprintf(stderr, "COM_write_bignum() failed\n");
	break;
      }
      if(COM_read_bn(str, n) < 0) {
	fprintf(stderr, "COM_read_bignum() failed\n");
	break;
      }
      BN_print_fp(stdout, n);
      putchar('\n');
    }

#ifdef STATS
    COM_print_stats(str);
#endif
    COM_close(str);
  }

  COM_ctx_close(ctx);
  UTL_free_ctx(utl_ctx);
}
