/* Copyright (C) 1999 Stanford University (dabo@cs.stanford.edu)
 * All rights reserved.
 *
 * This package is an implementation of the ITTC system written
 * by Michael Malkin, Tom Wu, and Dan Boneh (dabo@cs.stanford.edu).
 * 
 * This library is free for non-commercial use as long as
 * the following conditions are adhered to.  The following conditions
 * apply to all source code found in the src directory of this distribution.
 * Any source code found in other directories is subject to the copyright 
 * notice within that code.
 * 
 * Copyright belongs to Stanford University and as such any Copyright 
 * notices in the code are not to be removed.  This package may not be 
 * used in a commercial product without the explicit consent of the authors. 
 *
 * Free redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *    "This product includes software written by Mike Malkin, Tom Wu, 
 *     and Dan Boneh."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The license and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution license
 * [including the GNU Public License.]
 */

#include <stdio.h>
#include <signal.h>
#include "ITTC.h"

void *
square_body(void * s)
{
  BIGNUM * n;
  BIGNUM * r;
  BN_CTX * bnctx;
  int i;
  COM_STREAM * str = s;

  bnctx = BN_CTX_new();
  n = BN_new();
  r = BN_new();

  while((i = COM_read_bn(str, n)) > 0) {
    if(BN_is_zero(n))
      break;
    BN_print_fp(stdout, n);
    printf("^2 = ");
    BN_mul(r, n, n, bnctx);
    BN_print_fp(stdout, r);
    printf("\n");
    if(COM_write_bn(str, r) <= 0) {
      fprintf(stderr, "COM_write_bn() failed\n");
      break;
    }
  }
  if(i == 0)
    printf("EOF detected\n");
  else if(i < 0)
    fprintf(stderr, "COM_read_bn failed\n");
  return 0;
}

void * cube_body(void * s)
{
  BIGNUM * n;
  BIGNUM * r;
  BN_CTX * bnctx;
  int i;
  COM_STREAM * str = s;

  bnctx = BN_CTX_new();
  n = BN_new();
  r = BN_new();

  while((i = COM_read_bn(str, n)) > 0) {
    if(BN_is_zero(n))
      break;
    BN_print_fp(stdout, n);
    printf("^3 = ");
    BN_mul(r, n, n, bnctx);
    BN_mul(r, r, n, bnctx);
    BN_print_fp(stdout, r);
    printf("\n");
    if(COM_write_bn(str, r) <= 0) {
      fprintf(stderr, "COM_write_bn() failed\n");
      break;
    }
  }
  if(i == 0)
    printf("EOF detected\n");
  else if(i < 0)
    fprintf(stderr, "COM_read_bn failed\n");
  return 0;
}

UTL_CTX * utl_ctx;

void * square_accept(void * ctx)
{
  COM_STREAM * str;
  COM_INFO p_info;
  UTL_THREAD body_t;

  while(1) {
    str = COM_accept((COM_CTX *) ctx, 2);
    if(str != NULL) {
      printf("COM_accept() (square) returned\n");
      if(COM_get_peer_info(str, &p_info) < 0)
	fprintf(stderr, "Unable to get peer info\n");
      else
	UTL_monitor(utl_ctx, 0, "Peer: Type = %d, Num = %d, Vport = %d", COM_INFO_GET_TYPE(p_info), COM_INFO_GET_NUM(p_info), COM_INFO_GET_PORT(p_info));

      UTL_thread_create(&body_t, square_body, str);
    }
    else
      fprintf(stderr, "COM_accept (square) returned NULL\n");
  }
}

void * cube_accept(void * ctx)
{
  COM_STREAM * str;
  COM_INFO p_info;
  UTL_THREAD body_t;

  while(1) {
    str = COM_accept((COM_CTX *) ctx, 3);
    if(str != NULL) {
      printf("COM_accept() (cube) returned\n");
      if(COM_get_peer_info(str, &p_info) < 0)
	fprintf(stderr, "Unable to get peer info\n");
      else
	UTL_monitor(utl_ctx, 0, "Peer: Type = %d, Num = %d, Vport = %d", COM_INFO_GET_TYPE(p_info), COM_INFO_GET_NUM(p_info), COM_INFO_GET_PORT(p_info));

      UTL_thread_create(&body_t, cube_body, str);
    }
    else
      fprintf(stderr, "COM_accept (cube) returned NULL\n");
  }
}

int
main(argc, argv)
     int argc;
     char * argv[];
{
  COM_CTX * ctx;
  int num;
  UTL_THREAD sq_t, cb_t;

  if(argc < 3) {
    fprintf(stderr, "Usage: %s config-file server-num\n", argv[0]);
    exit(1);
  }

  ITTC_init();
  num = atoi(argv[2]);
  printf("Using config-file %s as server #%d\n", argv[1], num);
  utl_ctx = UTL_init_ctx();
  UTL_LoadConfigFile(utl_ctx, argv[1]);
  UTL_OpenLogFile(utl_ctx, COM_TYPE_SHARE_SERVER, num);
  UTL_MonitorInit(utl_ctx, COM_TYPE_SHARE_SERVER, num);

  if((ctx = COM_init(utl_ctx, COM_TYPE_SHARE_SERVER, num)) == NULL) {
    fprintf(stderr, "COM_init() failed\n");
    exit(1);
  }

  /*COM_ctx_set_flags(ctx, COM_FLAG_NONBLOCK);*/

  UTL_thread_create(&sq_t, square_accept, ctx);
  UTL_thread_create(&cb_t, cube_accept, ctx);
  UTL_thread_join(&sq_t);
  UTL_thread_join(&cb_t);
  UTL_thread_destroy(&sq_t);
  UTL_thread_destroy(&cb_t);

  COM_ctx_close(ctx);
  UTL_free_ctx(utl_ctx);
}
